@@ -77,12 +77,12 @@
 // Local Burp includes. Be sure to put all the system includes before these.
 #ifdef HAVE_WIN32
 	#include <windows.h>
-	#include "win32/compat/compat.h"
 #endif
 
 #include "burpconfig.h"
 
 #ifdef HAVE_WIN32
+	#include "win32/compat/compat.h"
 	#include "win32/winapi.h"
 	#include "winhost.h"
 #endif
@@ -139,6 +139,12 @@
 	#define __BIG_ENDIAN    BIG_ENDIAN
 	#define __LITTLE_ENDIAN	LITTLE_ENDIAN
 	#define __PDP_ENDIAN	PDP_ENDIAN
+#elif _AIX && __GNUC__
+	/* AIX is always big endian */
+	#define htobe64(x) (x)
+	#define htole64(x) __builtin_bswap64(x)
+	#define be64toh(x) (x)
+	#define le64toh(x) __builtin_bswap64(x)
 #endif
 
 #if !defined(htobe64) && defined(__GLIBC__) && __GLIBC__ <= 2 && __GLIBC_MINOR__ < 9
@@ -159,4 +165,15 @@
 	#endif
 #endif
 
+// This is the shape of the Windows VSS header structure.
+// It is size 20 on disk. Using sizeof(struct bsid) gives 24 in memory.
+struct bsid
+{
+	int32_t dwStreamId;
+	int32_t dwStreamAttributes;
+	int64_t Size;
+	int32_t dwStreamNameSize;
+};
+#define bsidsize        20
+
 #endif
