@@ -59,8 +59,11 @@
 #ifdef HAVE_LINUX_OS
 #include <sys/statfs.h>
 #endif
+#ifdef HAVE_SUN_OS
+#include <sys/statvfs.h>
+#endif
 
-static int (*send_file)(struct asfd *, struct FF_PKT *, struct conf **);
+static int (*my_send_file)(struct asfd *, struct FF_PKT *, struct conf **);
 
 // Initialize the find files "global" variables
 struct FF_PKT *find_files_init(
@@ -71,7 +74,7 @@ struct FF_PKT *find_files_init(
 	if(!(ff=(struct FF_PKT *)calloc_w(1, sizeof(struct FF_PKT), __func__))
 	  || linkhash_init())
 		return NULL;
-	send_file=callback;
+	my_send_file=callback;
 
 	return ff;
 }
@@ -283,7 +286,7 @@ static int file_size_match(struct FF_PKT *ff_pkt, struct conf **confs)
 }
 
 // Last checks before actually processing the file system entry.
-static int send_file_w(struct asfd *asfd, struct FF_PKT *ff, bool top_level, struct conf **confs)
+static int my_send_file_w(struct asfd *asfd, struct FF_PKT *ff, bool top_level, struct conf **confs)
 {
 	if(!file_is_included(confs, ff->fname, top_level)) return 0;
 
@@ -322,7 +325,7 @@ static int send_file_w(struct asfd *asfd, struct FF_PKT *ff, bool top_level, str
 		}
 	}
 
-	return send_file(asfd, ff, confs);
+	return my_send_file(asfd, ff, confs);
 }
 
 static int found_regular_file(struct asfd *asfd,
@@ -330,7 +333,7 @@ static int found_regular_file(struct asfd *asfd,
 	bool top_level)
 {
 	ff_pkt->type=FT_REG;
-	return send_file_w(asfd, ff_pkt, top_level, confs);
+	return my_send_file_w(asfd, ff_pkt, top_level, confs);
 }
 
 static int found_soft_link(struct asfd *asfd, struct FF_PKT *ff_pkt, struct conf **confs,
@@ -350,23 +353,40 @@ static int found_soft_link(struct asfd *asfd, struct FF_PKT *ff_pkt, struct conf
 		ff_pkt->link=buffer;	/* point to link */
 		ff_pkt->type=FT_LNK_S;	/* got a soft link */
 	}
-	return send_file_w(asfd, ff_pkt, top_level, confs);
+	return my_send_file_w(asfd, ff_pkt, top_level, confs);
 }
 
-static int fstype_excluded(struct asfd *asfd,
-	struct conf **confs, const char *fname)
+static int fstype_matches(struct asfd *asfd,
+	struct conf **confs, const char *fname, int inex)
 {
+#if defined(HAVE_LINUX_OS) \
+ || defined(HAVE_SUN_OS)
+	struct strlist *l;
 #if defined(HAVE_LINUX_OS)
 	struct statfs buf;
-	struct strlist *l;
 	if(statfs(fname, &buf))
+#elif defined(HAVE_SUN_OS)
+	struct statvfs buf;
+	if(statvfs(fname, &buf))
+#endif
 	{
 		logw(asfd, get_cntr(confs), "Could not statfs %s: %s\n",
 			fname, strerror(errno));
 		return -1;
 	}
-	for(l=get_strlist(confs[OPT_EXCFS]); l; l=l->next)
+	for(l=get_strlist(confs[inex]); l; l=l->next)
+#if defined(HAVE_LINUX_OS)
 		if(l->flag==buf.f_type)
+#elif defined(HAVE_SUN_OS)
+		if(strcmp(l->path,buf.f_basetype)==0)
+#endif
+			return -1;
+#elif defined(HAVE_WIN32)
+	char filesystem_name[MAX_PATH_UTF8 + 1];
+	if (win32_getfsname(fname, filesystem_name, sizeof(filesystem_name)))
+		return -1;
+	for(strlist *l=get_strlist(confs[inex]); l; l=l->next)
+		if(strcmp(l->path,filesystem_name)==0)
 			return -1;
 #endif
 	return 0;
@@ -518,7 +538,7 @@ static int found_directory(struct asfd *asfd,
 	windows_reparse_point_fiddling(ff_pkt);
 #endif
 
-	if(send_file_w(asfd, ff_pkt, top_level, confs))
+	if(my_send_file_w(asfd, ff_pkt, top_level, confs))
 		goto end;
 	if(ff_pkt->type==FT_REPARSE || ff_pkt->type==FT_JUNCTION)
 	{
@@ -534,16 +554,21 @@ static int found_directory(struct asfd *asfd,
 #endif
 		))
 	{
-		if(fstype_excluded(asfd, confs, ff_pkt->fname))
+		if(fstype_matches(asfd, confs, ff_pkt->fname, OPT_EXCFS)
+		  || (get_strlist(confs[OPT_INCFS])
+		     && !fstype_matches(asfd, confs, ff_pkt->fname, OPT_INCFS)))
 		{
-			ret=send_file_w(asfd, ff_pkt, top_level, confs);
+			if(top_level)
+				logw(asfd, get_cntr(confs),
+					"Skipping '%s' because of file system include or exclude.\n", fname);
+			ret=my_send_file_w(asfd, ff_pkt, top_level, confs);
 			goto end;
 		}
 		if(!top_level && !fs_change_is_allowed(confs, ff_pkt->fname))
 		{
 			ff_pkt->type=FT_NOFSCHG;
 			// Just backup the directory and return.
-			ret=send_file_w(asfd, ff_pkt, top_level, confs);
+			ret=my_send_file_w(asfd, ff_pkt, top_level, confs);
 			goto end;
 		}
 	}
@@ -557,7 +582,7 @@ static int found_directory(struct asfd *asfd,
 		case 0: break;
 		case 1:
 			ff_pkt->type=FT_NOOPEN;
-			ret=send_file_w(asfd, ff_pkt, top_level, confs);
+			ret=my_send_file_w(asfd, ff_pkt, top_level, confs);
 		default:
 			goto end;
 	}
@@ -607,7 +632,7 @@ static int found_other(struct asfd *asfd, struct FF_PKT *ff_pkt,
 		/* The only remaining are special (character, ...) files */
 		ff_pkt->type=FT_SPEC;
 	}
-	return send_file_w(asfd, ff_pkt, top_level, confs);
+	return my_send_file_w(asfd, ff_pkt, top_level, confs);
 }
 
 static int find_files(struct asfd *asfd, struct FF_PKT *ff_pkt, struct conf **confs,
@@ -623,7 +648,7 @@ static int find_files(struct asfd *asfd, struct FF_PKT *ff_pkt, struct conf **co
 #endif
 	{
 		ff_pkt->type=FT_NOSTAT;
-		return send_file_w(asfd, ff_pkt, top_level, confs);
+		return my_send_file_w(asfd, ff_pkt, top_level, confs);
 	}
 
 	if(S_ISREG(ff_pkt->statp.st_mode))
