@@ -12,6 +12,7 @@
 #include "../../handy.h"
 #include "../../iobuf.h"
 #include "../../log.h"
+#include "../../times.h"
 #include "json_input.h"
 #include "lline.h"
 #include "sel.h"
@@ -31,10 +32,6 @@ static enum action actg=ACTION_STATUS;
 
 static struct fzp *lfzp=NULL;
 
-// For switching between seeing 'last backup' and counter summary on the front
-// screen.
-static uint8_t toggle=0;
-
 #ifdef HAVE_NCURSES
 static void print_line_ncurses(const char *string, int row, int col)
 {
@@ -75,7 +72,7 @@ static void print_line(const char *string, int row, int col)
 
 static char *get_bu_str(struct bu *bu)
 {
-	static char ret[32];
+	static char ret[38];
 	if(!bu) snprintf(ret, sizeof(ret), "never");
 	else if(!bu->bno) snprintf(ret, sizeof(ret), "%s", bu->timestamp);
 	else snprintf(ret, sizeof(ret), "%07" PRIu64 " %s",
@@ -95,51 +92,8 @@ static void client_summary(struct cstat *cstat,
 	// Find the current backup.
 	cbu=bu_find_current(cstat->bu);
 
-	switch(cstat->run_status)
-	{
-		case RUN_STATUS_RUNNING:
-			if(toggle)
-			{
-				char f[64]="";
-				char b[64]="";
-				uint64_t p=0;
-				uint64_t t=0;
-				struct cntr *cntr=cstat->cntr;
-				struct cntr_ent *ent_gtotal=
-					cntr->ent[(uint8_t)CMD_GRAND_TOTAL];
-
-				t=ent_gtotal->count
-					+ent_gtotal->same
-					+ent_gtotal->changed;
-				if(ent_gtotal->phase1)
-					p=(t*100)/ent_gtotal->phase1;
-				snprintf(f, sizeof(f),
-					" %" PRIu64
-					"/%" PRIu64
-					" %" PRIu64
-					"%%",
-					t, ent_gtotal->phase1, p);
-				if(cntr->byte)
-					snprintf(b, sizeof(b), "%s",
-						bytes_to_human(cntr->byte));
-				snprintf(msg, sizeof(msg), fmt,
-					cstat->name,
-					run_status_to_str(cstat),
-					f, b);
-				break;
-			}
-			// Else fall through.
-		case RUN_STATUS_IDLE:
-		case RUN_STATUS_SERVER_CRASHED:
-		case RUN_STATUS_CLIENT_CRASHED:
-		default:
-			snprintf(msg, sizeof(msg), fmt,
-				cstat->name,
-				run_status_to_str(cstat),
-				" last backup: ",
-				get_bu_str(cbu));
-			break;
-	}
+	snprintf(msg, sizeof(msg), fmt, cstat->name, run_status_to_str(cstat),
+		" last backup: ", get_bu_str(cbu));
 
 	if(*msg) print_line(msg, row, col);
 }
@@ -391,7 +345,7 @@ static void detail(const char *cntrclient, char status, char phase, const char *
 static void screen_header_ncurses(const char *date, int l, int col)
 {
 	char v[32]="";
-	snprintf(v, sizeof(v), " burp monitor %s", VERSION);
+	snprintf(v, sizeof(v), " %s monitor %s", PACKAGE_TARNAME, VERSION);
 	print_line(v, 0-TOP_SPACE, col);
 	mvprintw(0, col-l-1, date);
 }
@@ -401,9 +355,13 @@ static void screen_header_stdout(const char *date, int l, int col)
 {
 	size_t c=0;
 	char spaces[512]="";
-	stdout_asfd->write_str(stdout_asfd, CMD_GEN, "\n burp status");
+	char msg[64]="";
+	snprintf(msg, sizeof(msg), " %s status", PACKAGE_TARNAME);
+
+	stdout_asfd->write_str(stdout_asfd, CMD_GEN, "\n");
+	stdout_asfd->write_str(stdout_asfd, CMD_GEN, msg);
 	for(c=0;
-	  c<(col-strlen(" burp status")-l-1)
+	  c<(col-strlen(msg)-l-1)
 		&& c<sizeof(spaces)-1; c++)
 			spaces[c]=' ';
 	spaces[c]='\0';
@@ -419,8 +377,7 @@ static void screen_header(int col)
 #ifdef UTEST
 	date="1977-10-02 00:10:20";
 #else
-	time_t t=time(NULL);
-	date=getlocaldatestr(t);
+	date=gettimenow();
 #endif
 	l=strlen(date);
 #ifdef HAVE_NCURSES
@@ -466,6 +423,7 @@ static const char *logop_to_text(uint16_t logop)
 		case BU_STATS_BACKUP:	return "Backup stats";
 		case BU_STATS_RESTORE:	return "Restore stats";
 		case BU_STATS_VERIFY:	return "Verify stats";
+		case BU_LIVE_COUNTERS:	return "Live counters";
 		default: return "";
 	}
 }
@@ -485,13 +443,19 @@ static void print_logs_list_line(struct sel *sel,
 #endif
 }
 
-static void client_and_status(struct sel *sel, int *x, int col)
+static void print_client(struct sel *sel, int *x, int col)
 {
-	char msg[1024];
+	char msg[1024]="";
 	snprintf(msg, sizeof(msg), "Client: %s", sel->client->name);
 //		sel->client->cntr->ent[CMD_FILE]->phase1,
 //		sel->client->cntr->ent[CMD_FILE]->count);
 	print_line(msg, (*x)++, col);
+}
+
+static void client_and_status(struct sel *sel, int *x, int col)
+{
+	char msg[1024]="";
+	print_client(sel, x, col);
 	snprintf(msg, sizeof(msg),
 		"Status: %s", run_status_to_str(sel->client));
 	print_line(msg, (*x)++, col);
@@ -554,6 +518,7 @@ static int selindex_from_lline(struct sel *sel)
 
 static void print_logs_list(struct sel *sel, int *x, int col)
 {
+	print_logs_list_line(sel, BU_LIVE_COUNTERS, x, col);
 	print_logs_list_line(sel, BU_MANIFEST, x, col);
 	print_logs_list_line(sel, BU_LOG_BACKUP, x, col);
 	print_logs_list_line(sel, BU_LOG_RESTORE, x, col);
@@ -571,7 +536,7 @@ static void update_screen_clients(struct sel *sel, int *x, int col,
 #endif
 	struct cstat *c;
 	int star_printed=0;
-	int max_cname=28*((float)col/100);
+	int max_cname=23*((float)col/100);
 #ifdef HAVE_NCURSES
 	if(actg==ACTION_STATUS_SNAPSHOT)
 #endif
@@ -605,7 +570,43 @@ static void update_screen_clients(struct sel *sel, int *x, int col,
 	if(!star_printed) sel->client=sel->clist;
 }
 
-static void update_screen_backups(struct sel *sel, int *x, int col,
+static char *get_extradesc(struct bu *b, struct cntr *cntrs)
+{
+	char *extradesc=NULL;
+	struct cntr *cntr=NULL;
+	if(b->flags & BU_CURRENT)
+	{
+		extradesc=strdup_w(" (current)", __func__);
+	}
+	else if(b->flags & BU_WORKING)
+	{
+		extradesc=strdup_w(" (working)", __func__);
+	}
+	else if(b->flags & BU_FINISHING)
+	{
+		extradesc=strdup_w(" (finishing)", __func__);
+	}
+	else
+	{
+		extradesc=strdup_w("", __func__);
+	}
+
+	for(cntr=cntrs; cntr; cntr=cntr->next)
+	{
+		char phase[32]="";
+		if(cntr->bno==b->bno)
+		{
+			snprintf(phase, sizeof(phase),
+				" %s, pid: %d",
+				cntr_status_to_str(cntr), cntr->pid);
+			if(astrcat(&extradesc, phase, __func__))
+				return NULL;
+		}
+	}
+	return extradesc;
+}
+
+static int update_screen_backups(struct sel *sel, int *x, int col,
 	int winmin, int winmax)
 {
 #ifdef HAVE_NCURSES
@@ -614,9 +615,9 @@ static void update_screen_backups(struct sel *sel, int *x, int col,
 	struct bu *b;
 	char msg[1024]="";
 	int star_printed=0;
-	const char *extradesc=NULL;
 	for(b=sel->client->bu; b; b=b->next)
 	{
+		char *extradesc=NULL;
 #ifdef HAVE_NCURSES
 		if(actg==ACTION_STATUS)
 		{
@@ -626,19 +627,15 @@ static void update_screen_backups(struct sel *sel, int *x, int col,
 		}
 #endif
 
-		if(b->flags & BU_CURRENT)
-			extradesc=" (current)";
-		else if(b->flags & BU_WORKING)
-			extradesc=" (working)";
-		else if(b->flags & BU_FINISHING)
-			extradesc=" (finishing)";
-		else extradesc="";
+		if(!(extradesc=get_extradesc(b, sel->client->cntrs)))
+			return -1;
 
 		snprintf(msg, sizeof(msg), "%s %s%s",
 				b==sel->client->bu?"Backup list:":
 				"            ",
 				get_bu_str(b),
 				extradesc);
+		free_w(&extradesc);
 		print_line(msg, (*x)++, col);
 #ifdef HAVE_NCURSES
 		if(actg==ACTION_STATUS && sel->backup==b)
@@ -649,6 +646,7 @@ static void update_screen_backups(struct sel *sel, int *x, int col,
 #endif
 	}
 	if(!star_printed) sel->backup=sel->client->bu;
+	return 0;
 }
 
 static void update_screen_live_counter_table(struct cntr_ent *e,
@@ -690,16 +688,18 @@ static void update_screen_live_counter_single(struct cntr_ent *e,
 	print_line(msg, (*x)++, col);
 }
 
-static void update_screen_live_counters(struct cstat *client, int *x, int col)
+static void update_screen_live_counters(struct cntr *cntr, int *x, int col)
 {
 	char msg[128]="";
 	struct cntr_ent *e;
-	struct cntr *cntr=client->cntr;
 	time_t start=(time_t)cntr->ent[(uint8_t)CMD_TIMESTAMP]->count;
 	time_t end=(time_t)cntr->ent[(uint8_t)CMD_TIMESTAMP_END]->count;
 	struct cntr_ent *gtotal=cntr->ent[(uint8_t)CMD_GRAND_TOTAL];
 
 	print_line("", (*x)++, col);
+	snprintf(msg, sizeof(msg), "       PID: %d (%s)",
+		cntr->pid, cntr_status_to_str(cntr));
+	print_line(msg, (*x)++, col);
 	snprintf(msg, sizeof(msg), "Start time: %s", getdatestr(start));
 	print_line(msg, (*x)++, col);
 	snprintf(msg, sizeof(msg), "  End time: %s", getdatestr(end));
@@ -707,17 +707,34 @@ static void update_screen_live_counters(struct cstat *client, int *x, int col)
 	snprintf(msg, sizeof(msg), "Time taken: %s", time_taken(end-start));
 	print_line(msg, (*x)++, col);
 	table_header(x, col);
-	for(e=client->cntr->list; e; e=e->next)
+	for(e=cntr->list; e; e=e->next)
 		update_screen_live_counter_table(e, x, col);
 	print_line("", (*x)++, col);
-	snprintf(msg, sizeof(msg), "%19s: %" PRIu64 "%%", "Percentage complete",
-	  ((gtotal->count+gtotal->same+gtotal->changed)*100)/gtotal->phase1);
-	print_line(msg, (*x)++, col);
+
+	if(gtotal->phase1)
+	{
+		snprintf(msg, sizeof(msg),
+			"%19s: %" PRIu64 "%%", "Percentage complete",
+			((gtotal->count+gtotal->same+gtotal->changed)*100)/gtotal->phase1);
+		print_line(msg, (*x)++, col);
+	}
 	print_line("", (*x)++, col);
-	for(e=client->cntr->list; e; e=e->next)
+	for(e=cntr->list; e; e=e->next)
 		update_screen_live_counter_single(e, x, col);
 }
 
+static void update_screen_live_counters_w(struct sel *sel, int *x, int col)
+{
+	struct cstat *client=sel->client;
+	struct cntr *cntr=NULL;
+	for(cntr=client->cntrs; cntr; cntr=cntr->next)
+	{
+		if(sel->backup
+		  && sel->backup->bno==cntr->bno)
+			update_screen_live_counters(cntr, x, col);
+	}
+}
+
 static void update_screen_view_log(struct sel *sel, int *x, int col,
 	int winmin, int winmax)
 {
@@ -731,9 +748,8 @@ static void update_screen_view_log(struct sel *sel, int *x, int col,
 
 	if(sel->client
 	  && sel->backup
-	  && (sel->backup->flags & (BU_WORKING|BU_FINISHING))
-	  && (sel->logop & BU_STATS_BACKUP))
-		return update_screen_live_counters(sel->client, x, col);
+	  && (sel->logop & BU_LIVE_COUNTERS))
+		return update_screen_live_counters_w(sel, x, col);
 
 	for(l=sel->llines; l; l=l->next)
 	{
@@ -870,7 +886,8 @@ static int update_screen(struct sel *sel)
 			update_screen_clients(sel, &x, col, winmin, winmax);
 			break;
 		case PAGE_BACKUP_LIST:
-			update_screen_backups(sel, &x, col, winmin, winmax);
+			if(update_screen_backups(sel, &x, col, winmin, winmax))
+				return -1;
 			break;
 		case PAGE_BACKUP_LOGS:
 			print_logs_list(sel, &x, col);
@@ -921,32 +938,27 @@ static int request_status(struct asfd *asfd,
 				lname="verify";
 			else if(sel->logop & BU_MANIFEST)
 				lname="manifest";
-			else if(sel->logop & BU_STATS_BACKUP)
-			{
-			// Hack so that it does not request the logs for live
-			// counters.
-			// FIX THIS: need to do something similar for
-			// restore/verify.
-				if(!sel->backup) break;
-				if(sel->client
-				  && sel->client->run_status==RUN_STATUS_RUNNING
-				  && sel->backup->flags
-					& (BU_WORKING|BU_FINISHING))
-				{
-					// Make sure a request is sent, so that
-					// the counters update.
-					snprintf(buf, sizeof(buf),
-						"c:%s:b:%" PRIu64 "\n",
-						client, sel->backup->bno);
-					break;
-				}
-				else
-					lname="backup_stats";
-			}
 			else if(sel->logop & BU_STATS_RESTORE)
 				lname="restore_stats";
 			else if(sel->logop & BU_STATS_VERIFY)
 				lname="verify_stats";
+			else if(sel->logop & BU_STATS_BACKUP)
+				lname="backup_stats";
+			else if(sel->logop & BU_LIVE_COUNTERS)
+			{
+				// Hack so that it does not request the logs
+				// for live counters.
+				if(!sel->backup
+				  || !sel->client
+				  || !sel->client->cntrs)
+					break;
+				// Make sure a request is sent, so that the
+				// counters update.
+				snprintf(buf, sizeof(buf),
+					"c:%s:b:%" PRIu64 "\n",
+					client, sel->backup->bno);
+				break;
+			}
 
 			if(sel->backup && lname)
 				snprintf(buf, sizeof(buf),
@@ -1074,7 +1086,7 @@ static void up_logs(struct sel *sel)
 {
 	int i=0;
 	uint16_t sh=sel->logop;
-	for(i=0; sh>BU_MANIFEST && i<16; i++)
+	for(i=0; sh>BU_LIVE_COUNTERS && i<16; i++)
 	{
 		sh=sh>>1;
 		if(sh & sel->backup->flags)
@@ -1247,11 +1259,6 @@ static int parse_stdin_data(struct asfd *asfd, struct sel *sel)
 		case 'q':
 		case 'Q':
 			return 1;
-		case 't':
-		case 'T':
-			if(toggle) toggle=0;
-			else toggle=1;
-			break;
 		case KEY_UP:
 		case 'k':
 		case 'K':
@@ -1300,9 +1307,16 @@ static int parse_data(struct asfd *asfd, struct sel *sel)
 	{
 		// 0 means carry on.
 		// 1 means it got to the end of the JSON statement.
+		// 2 means it got to the end but had warnings.
 		// Anything else means an error.
 		case 0: return 0;
 		case 1: return 0;
+		case 2:
+		{
+			// If we had a warning exit straight away. For example,
+			// if they specified '-C non-existent-client'.
+			return -1;
+		}
 		default: return -1;
 	}
 }
@@ -1350,7 +1364,7 @@ int status_client_ncurses_main_loop(struct async *as,
 				else if(sel->client)
 					req=sel->client->name;
 			}
-			if(request_status(sfd, req, sel))
+			if(request_status(sfd, req?req:"", sel))
 				goto error;
 
 			// We only want to start on the client the user gave to
@@ -1452,14 +1466,23 @@ static pid_t fork_monitor(int *csin, int *csout, struct conf **confs)
 	char *args[12];
 	char procpath[32];
 	char buf[PATH_MAX];
+	char *monitor_exe;
 
+	monitor_exe=get_string(confs[OPT_MONITOR_EXE]);
 	snprintf(procpath, sizeof(procpath), "/proc/%d/exe", getpid());
-	if(!readlink_w(procpath, buf, sizeof(buf)))
+	if(monitor_exe && is_reg_lstat(monitor_exe)>0)
+		args[a++]=monitor_exe;
+	else if(!readlink_w(procpath, buf, sizeof(buf)))
 		args[a++]=(char *)buf;
 	else if(is_reg_lstat(prog_long)>0)
 		args[a++]=(char *)prog_long;
 	else
-		args[a++]=(char *)"/usr/sbin/burp";
+	{
+		static char p[64]="";
+		snprintf(p, sizeof(p), "/usr/sbin/%s", PACKAGE_TARNAME);
+		logp("Using fallback monitor path: %s\n", p);
+		args[a++]=p;
+	}
 
 	args[a++]=(char *)"-c";
 	args[a++]=get_string(confs[OPT_CONFFILE]);
@@ -1483,11 +1506,11 @@ int status_client_ncurses_init(enum action act)
 	return 0;
 }
 
-static void show_loglines(struct lline *llines)
+static void show_loglines(struct lline *llines, const char *prefix)
 {
 	struct lline *l;
 	for(l=llines; l; l=l->next)
-		logp("%s\n", l->line);
+		logp("%s%s\n", prefix, l->line);
 }
 
 int status_client_ncurses(struct conf **confs)
@@ -1501,6 +1524,7 @@ int status_client_ncurses(struct conf **confs)
 	struct asfd *so_asfd=NULL;
 	struct sel *sel=NULL;
 	struct lline *llines=NULL;
+	struct lline *wlines=NULL;
 
 	if(json_input_init())
 		goto end;
@@ -1546,12 +1570,15 @@ end:
 		ncurses_free();
 #endif
 	llines=json_input_get_loglines();
+	wlines=json_input_get_warnings();
 	if(ret)
 	{
-		show_loglines(llines);
+		show_loglines(llines, "");
+		show_loglines(wlines, "WARNING: ");
 		logp("%s exiting with error: %d\n", __func__, ret);
 	}
-	llines_free(&llines);
+	json_input_clear_loglines();
+	json_input_clear_warnings();
 	json_input_free();
 	fzp_close(&lfzp);
 	async_asfd_free_all(&as);
