@@ -429,19 +429,29 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	case OPT_ADDRESS:
 	  return sc_str(c[o], 0, 0, "address");
 	case OPT_PORT:
-	  return sc_str(c[o], 0, 0, "port");
+	  return sc_lst(c[o], 0, 0, "port");
+	case OPT_PORT_BACKUP:
+	  return sc_int(c[o], 0, 0, "port_backup");
+	case OPT_PORT_RESTORE:
+	  return sc_int(c[o], 0, 0, "port_restore");
+	case OPT_PORT_VERIFY:
+	  return sc_int(c[o], 0, 0, "port_verify");
+	case OPT_PORT_LIST:
+	  return sc_int(c[o], 0, 0, "port_list");
+	case OPT_PORT_DELETE:
+	  return sc_int(c[o], 0, 0, "port_delete");
 	case OPT_STATUS_ADDRESS:
 	  return sc_str(c[o], 0, 0, "status_address");
 	case OPT_STATUS_PORT:
-	  return sc_str(c[o], 0, 0, "status_port");
+	  return sc_lst(c[o], 0, 0, "status_port");
 	case OPT_SSL_DHFILE:
 	  return sc_str(c[o], 0, 0, "ssl_dhfile");
 	case OPT_MAX_CHILDREN:
-	  return sc_int(c[o], 5, 0, "max_children");
+	  return sc_lst(c[o], 0, 0, "max_children");
 	case OPT_MAX_STATUS_CHILDREN:
-	  return sc_int(c[o], 5, 0, "max_status_children");
+	  return sc_lst(c[o], 0, 0, "max_status_children");
 	case OPT_CLIENT_LOCKDIR:
-	  return sc_str(c[o], 0, 0, "client_lockdir");
+	  return sc_str(c[o], 0, CONF_FLAG_CC_OVERRIDE, "client_lockdir");
 	case OPT_UMASK:
 	  return sc_mod(c[o], 0022, 0, "umask");
 	case OPT_MAX_HARDLINKS:
@@ -457,13 +467,18 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	case OPT_CA_SERVER_NAME:
 	  return sc_str(c[o], 0, 0, "ca_server_name");
 	case OPT_CA_BURP_CA:
-	  return sc_str(c[o], 0, 0, "ca_burp_ca");
+	  return sc_str(c[o], 0, 0, "ca_" PACKAGE_TARNAME "_ca");
         case OPT_CA_CRL:
           return sc_str(c[o], 0, 0, "ca_crl");
         case OPT_CA_CRL_CHECK:
           return sc_int(c[o], 0, 0, "ca_crl_check");
+	case OPT_RBLK_MEMORY_MAX:
+	  return sc_u64(c[o], 256*1024*1024, // 256 Mb.
+		CONF_FLAG_CC_OVERRIDE, "rblk_memory_max");
 	case OPT_MONITOR_LOGFILE:
 	  return sc_str(c[o], 0, 0, "monitor_logfile");
+	case OPT_MONITOR_EXE:
+	  return sc_str(c[o], 0, 0, "monitor_exe");
 	case OPT_CNAME:
 	  return sc_str(c[o], 0, 0, "cname");
 	case OPT_CNAME_LOWERCASE:
@@ -496,6 +511,8 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	  return sc_str(c[o], 0, 0, "backup2");
 	case OPT_RESTOREPREFIX:
 	  return sc_str(c[o], 0, CONF_FLAG_INCEXC_RESTORE, "restoreprefix");
+	case OPT_STRIP_FROM_PATH:
+	  return sc_str(c[o], 0, CONF_FLAG_INCEXC_RESTORE, "stripfrompath");
 	case OPT_BROWSEFILE:
 	  return sc_str(c[o], 0, 0, "browsefile");
 	case OPT_BROWSEDIR:
@@ -630,6 +647,9 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	case OPT_LIBRSYNC:
 	  return sc_int(c[o], 1,
 		CONF_FLAG_CC_OVERRIDE, "librsync");
+	case OPT_LIBRSYNC_MAX_SIZE:
+	  return sc_u64(c[o], 0,
+		CONF_FLAG_CC_OVERRIDE, "librsync_max_size");
 	case OPT_COMPRESSION:
 	  return sc_int(c[o], 9,
 		CONF_FLAG_CC_OVERRIDE, "compression");
@@ -747,6 +767,9 @@ static int reset_conf(struct conf **c, enum conf_opt o)
 	case OPT_EXCFS:
 	  return sc_lst(c[o], 0,
 		CONF_FLAG_INCEXC|CONF_FLAG_STRLIST_SORTED, "exclude_fs");
+	case OPT_INCFS:
+	  return sc_lst(c[o], 0,
+		CONF_FLAG_INCEXC|CONF_FLAG_STRLIST_SORTED, "include_fs");
 	case OPT_EXCOM:
 	  return sc_lst(c[o], 0,
 		CONF_FLAG_INCEXC|CONF_FLAG_STRLIST_SORTED, "exclude_comp");
