@@ -127,11 +127,18 @@ enum conf_opt
 	OPT_CA_CRL,
 	OPT_PASSWORD_CHECK,
 	OPT_MANUAL_DELETE,
+	OPT_RBLK_MEMORY_MAX,
 	OPT_MONITOR_LOGFILE, // An ncurses client option, from command line.
 	OPT_MONITOR_BROWSE_CACHE,
+	OPT_MONITOR_EXE,
 
 	// Client options.
 	OPT_CNAME, // set on the server when client connects
+	OPT_PORT_BACKUP,
+	OPT_PORT_RESTORE,
+	OPT_PORT_VERIFY,
+	OPT_PORT_LIST,
+	OPT_PORT_DELETE,
 	OPT_PASSWORD, // also a clientconfdir option
 	OPT_PASSWD, // also a clientconfdir option
 	OPT_ENABLED, // also a clientconfdir option
@@ -155,6 +162,7 @@ enum conf_opt
 	OPT_INCREG, // include (regular expression)
 	OPT_EXCREG, // exclude (regular expression)
 	OPT_EXCFS, // exclude filesystems
+	OPT_INCFS, // include filesystems
 	OPT_EXCOM, // exclude from compression
 	OPT_INCGLOB, // include (glob expression)
 	OPT_CROSS_ALL_FILESYSTEMS,
@@ -174,6 +182,7 @@ enum conf_opt
 	// These are to do with restore.
 	OPT_OVERWRITE,
 	OPT_STRIP,
+	OPT_STRIP_FROM_PATH,
 	OPT_BACKUP,
 	OPT_BACKUP2, // For diffs.
 	OPT_RESTOREPREFIX,
@@ -228,6 +237,7 @@ enum conf_opt
 
 	OPT_WORKING_DIR_RECOVERY_METHOD,
 	OPT_LIBRSYNC,
+	OPT_LIBRSYNC_MAX_SIZE,
 
 	OPT_COMPRESSION,
 	OPT_VERSION_WARN,
