@@ -9,7 +9,7 @@
 #include "pathcmp.h"
 #include "prepend.h"
 #include "strlist.h"
-#include "server/timestamp.h"
+#include "times.h"
 #include "client/glob_windows.h"
 #include "conffile.h"
 
@@ -62,7 +62,7 @@ static int remove_quotes(const char *f, char **v, char quote)
 }
 
 // Get field and value pair.
-int conf_get_pair(char buf[], char **f, char **v)
+int conf_get_pair(char buf[], char **f, char **v, int *r)
 {
 	char *cp=NULL;
 	char *eq=NULL;
@@ -80,7 +80,11 @@ int conf_get_pair(char buf[], char **f, char **v)
 	*eq='\0';
 
 	// Strip white space from before the equals sign.
-	for(cp=eq-1; *cp && isspace(*cp); cp--) *cp='\0';
+	for(cp=eq-1; *cp && (isspace(*cp) || *cp == ':'); cp--)
+	{
+		if(*cp == ':') *r=1;
+		*cp='\0';
+	}
 	// Skip white space after the equals sign.
 	for(cp=eq+1; *cp && isspace(*cp); cp++) { }
 	*v=cp;
@@ -106,33 +110,6 @@ int conf_get_pair(char buf[], char **f, char **v)
 	return 0;
 }
 
-static int path_checks(const char *path, const char *err_msg)
-{
-	const char *p=NULL;
-	for(p=path; *p; p++)
-	{
-		if(*p!='.' || *(p+1)!='.') continue;
-		if((p==path || *(p-1)=='/') && (*(p+2)=='/' || !*(p+2)))
-		{
-			logp("%s", err_msg);
-			return -1;
-		}
-	}
-// This is being run on the server too, where you can enter paths for the
-// clients, so need to allow windows style paths for windows and unix.
-	if((!isalpha(*path) || *(path+1)!=':')
-#ifndef HAVE_WIN32
-	  // Windows does not need to check for unix style paths.
-	  && *path!='/'
-#endif
-	)
-	{
-		logp("%s", err_msg);
-		return -1;
-	}
-	return 0;
-}
-
 static int conf_error(const char *conf_path, int line)
 {
 	logp("%s: parse error on line %d\n", conf_path, line);
@@ -177,29 +154,38 @@ struct fstype
 	uint64_t flag;
 };
 
+// Sorted in magic number order.
 static struct fstype fstypes[]={
-	{ "debugfs",		0x64626720 },
 	{ "devfs",		0x00001373 },
 	{ "devpts",		0x00001CD1 },
+	{ "smbfs",		0x0000517B },
+	{ "nfs",		0x00006969 },
+	{ "romfs",		0x00007275 },
+	{ "iso9660",		0x00009660 },
 	{ "devtmpfs",		0x00009FA0 },
-	{ "ecryptfs",		0x0000f15f },
+	{ "proc",		0x00009FA0 },
+	{ "usbdevfs",		0x00009FA2 },
 	{ "ext2",		0x0000EF53 },
 	{ "ext3",		0x0000EF53 },
 	{ "ext4",		0x0000EF53 },
-	{ "iso9660",		0x00009660 },
-	{ "jfs",		0x3153464A },
-	{ "nfs",		0x00006969 },
-	{ "ntfs",		0x5346544E },
-	{ "proc",		0x00009fa0 },
-	{ "reiserfs",		0x52654973 },
-	{ "securityfs",		0x73636673 },
-	{ "sysfs",		0x62656572 },
-	{ "smbfs",		0x0000517B },
-	{ "usbdevfs",		0x00009fa2 },
-	{ "xfs",		0x58465342 },
-	{ "ramfs",		0x858458f6 },
-	{ "romfs",		0x00007275 },
+	{ "ecryptfs",		0x0000F15F },
+	{ "cgroup",		0x0027E0EB },
 	{ "tmpfs",		0x01021994 },
+	{ "zfs",		0x2FC12FC1 },
+	{ "jfs",		0x3153464A },
+	{ "autofs",		0x42494E4D },
+	{ "reiserfs",		0x52654973 },
+	{ "ntfs",		0x5346544E },
+	{ "xfs",		0x58465342 },
+	{ "sysfs",		0x62656572 },
+	{ "debugfs",		0x64626720 },
+	{ "fusectl", 		0x65735543 },
+	{ "fuse.lxcfs",		0x65735546 },
+	{ "securityfs",		0x73636673 },
+	{ "ramfs",		0x858458F6 },
+	{ "btrfs",		0x9123683E },
+	{ "hugetlbfs",		0x958458F6 },
+	{ "cifs",		0xFF534D42 },
 	{ NULL,			0 },
 };
 /* Use this C code to figure out what f_type gets set to.
@@ -259,6 +245,7 @@ static int get_compression(const char *v)
 static int load_conf_field_and_value(struct conf **c,
 	const char *f, // field
 	const char *v, // value
+	int reset, // reset flag
 	const char *conf_path,
 	int line)
 {
@@ -323,6 +310,7 @@ static int load_conf_field_and_value(struct conf **c,
 					return set_e_recovery_method(c[i],
 						str_to_recovery_method(v));
 				case CT_STRLIST:
+					if (reset) set_strlist(c[i], 0);
 					return add_to_strlist(c[i], v,
 					  !strcmp(c[i]->field, "include"));
 				case CT_E_RSHASH:
@@ -377,6 +365,8 @@ static int deal_with_dot_inclusion(const char *conf_path,
 	for(i=0; (unsigned int)i<globbuf.gl_pathc; i++)
 		if((ret=conf_load_lines_from_file(globbuf.gl_pathv[i], confs)))
 			goto end;
+
+	globfree(&globbuf);
 #else
 	ret=conf_load_lines_from_file(*extrafile, confs);
 #endif
@@ -390,6 +380,7 @@ static int conf_parse_line(struct conf **confs, const char *conf_path,
 	char buf[], int line)
 {
 	int ret=-1;
+	int r=0;
 	char *f=NULL; // field
 	char *v=NULL; // value
 	char *extrafile=NULL;
@@ -408,9 +399,9 @@ static int conf_parse_line(struct conf **confs, const char *conf_path,
 		goto end;
 	}
 
-	if(conf_get_pair(buf, &f, &v)) goto end;
+	if(conf_get_pair(buf, &f, &v, &r)) goto end;
 	if(f && v
-	  && load_conf_field_and_value(confs, f, v, conf_path, line))
+	  && load_conf_field_and_value(confs, f, v, r, conf_path, line))
 		goto end;
 	ret=0;
 end:
@@ -424,18 +415,30 @@ static void conf_problem(const char *conf_path, const char *msg, int *r)
 	(*r)--;
 }
 
+static void burp_ca_conf_problem(const char *conf_path,
+	const char *field, int *r)
+{
+	char msg[128]="";
+	snprintf(msg, sizeof(msg), "ca_%s_ca set, but %s not set\n",
+		PACKAGE_TARNAME, field);
+	conf_problem(conf_path, msg, r);
+}
+
 #ifdef HAVE_IPV6
 // These should work for IPv4 connections too.
 #define DEFAULT_ADDRESS_MAIN	"::"
-#define DEFAULT_ADDRESS_STATUS	"::1"
 #else
 // Fall back to IPv4 address if IPv6 is not compiled in.
 #define DEFAULT_ADDRESS_MAIN	"0.0.0.0"
-#define DEFAULT_ADDRESS_STATUS	"127.0.0.1"
 #endif
 
+#define DEFAULT_ADDRESS_STATUS	"localhost"
+
 static int server_conf_checks(struct conf **c, const char *path, int *r)
 {
+	if(!get_strlist(c[OPT_PORT]))
+		conf_problem(path, "port unset", r);
+
 	// FIX THIS: Most of this could be done by flags.
 	if(!get_string(c[OPT_ADDRESS])
 	  && set_string(c[OPT_ADDRESS], DEFAULT_ADDRESS_MAIN))
@@ -456,20 +459,13 @@ static int server_conf_checks(struct conf **c, const char *path, int *r)
 	if(!get_string(c[OPT_STATUS_ADDRESS])
 	  && set_string(c[OPT_STATUS_ADDRESS], DEFAULT_ADDRESS_STATUS))
 			return -1;
-	if(!get_string(c[OPT_STATUS_PORT])) // carry on if not set.
+	if(!get_strlist(c[OPT_STATUS_PORT])) // carry on if not set.
 		logp("%s: status_port unset", path);
-	if(!get_int(c[OPT_MAX_CHILDREN]))
-		conf_problem(path, "max_children unset", r);
-	if(!get_int(c[OPT_MAX_STATUS_CHILDREN]))
-		conf_problem(path, "max_status_children unset", r);
 	if(!get_strlist(c[OPT_KEEP]))
 		conf_problem(path, "keep unset", r);
 	if(get_int(c[OPT_MAX_HARDLINKS])<2)
 		conf_problem(path, "max_hardlinks too low", r);
-	if(get_int(c[OPT_MAX_CHILDREN])<=0)
-		conf_problem(path, "max_children too low", r);
-	if(get_int(c[OPT_MAX_STATUS_CHILDREN])<=0)
-		conf_problem(path, "max_status_children too low", r);
+
 	if(get_int(c[OPT_MAX_STORAGE_SUBDIRS])<=1000)
 		conf_problem(path, "max_storage_subdirs too low", r);
 	if(!get_string(c[OPT_TIMESTAMP_FORMAT])
@@ -490,7 +486,8 @@ static int server_conf_checks(struct conf **c, const char *path, int *r)
 		}
 		if(!get_string(c[OPT_CA_BURP_CA]))
 		{
-			logp("ca_conf set, but ca_burp_ca not set\n");
+			logp("ca_conf set, but ca_%s_ca not set\n",
+				PACKAGE_TARNAME);
 			ca_err++;
 		}
 		if(!get_string(c[OPT_SSL_DHFILE]))
@@ -517,9 +514,11 @@ static int server_conf_checks(struct conf **c, const char *path, int *r)
 	}
 	if(get_string(c[OPT_MANUAL_DELETE]))
 	{
-		if(path_checks(get_string(c[OPT_MANUAL_DELETE]),
-			"ERROR: Please use an absolute manual_delete path.\n"))
-				return -1;
+		if(!is_absolute(get_string(c[OPT_MANUAL_DELETE])))
+		{
+			logp("ERROR: Please use an absolute manual_delete path.\n");
+			return -1;
+		}
 	}
 
 	return 0;
@@ -631,19 +630,23 @@ static int get_fqdn(struct conf **c)
 	{
 		logp("getaddrinfo in %s: %s\n", __func__,
 			gai_strerror(gai_result));
-		goto end;
+		logp("Using %s\n", hostname);
+		if(!(fqdn=strdup_w(hostname, __func__)))
+			goto end;
 	}
-
-	//for(p=info; p; p=p->ai_next)
-	// Just use the first one.
-	if(!info)
+	else
 	{
-		logp("Got no hostname in %s\n", __func__);
-		goto end;
+		//for(p=info; p; p=p->ai_next)
+		// Just use the first one.
+		if(!info)
+		{
+			logp("Got no hostname in %s\n", __func__);
+			goto end;
+		}
+		if(!(fqdn=strdup_w(info->ai_canonname, __func__)))
+			goto end;
 	}
 
-	if(!(fqdn=strdup_w(info->ai_canonname, __func__)))
-		goto end;
 	mangle_cname(&fqdn, c);
 
 	if(set_string(c[OPT_CNAME], fqdn))
@@ -678,6 +681,18 @@ static int general_conf_checks(struct conf **c, const char *path, int *r)
 static int client_conf_checks(struct conf **c, const char *path, int *r)
 {
 	const char *autoupgrade_os=get_string(c[OPT_AUTOUPGRADE_OS]);
+
+	if(!get_int(c[OPT_PORT_BACKUP]))
+		conf_problem(path, "port_backup unset", r);
+	if(!get_int(c[OPT_PORT_RESTORE]))
+		conf_problem(path, "port_restore unset", r);
+	if(!get_int(c[OPT_PORT_VERIFY]))
+		conf_problem(path, "port_verify unset", r);
+	if(!get_int(c[OPT_PORT_LIST]))
+		conf_problem(path, "port_list unset", r);
+	if(!get_int(c[OPT_PORT_DELETE]))
+		conf_problem(path, "port_delete unset", r);
+
 	if(!get_string(c[OPT_CNAME]))
 	{
 		if(get_cname_from_ssl_cert(c)) return -1;
@@ -698,7 +713,7 @@ static int client_conf_checks(struct conf **c, const char *path, int *r)
 	}
 	if(!get_string(c[OPT_SERVER]))
 		conf_problem(path, "server unset", r);
-	if(!get_string(c[OPT_STATUS_PORT])) // carry on if not set.
+	if(!get_strlist(c[OPT_STATUS_PORT])) // carry on if not set.
 		logp("%s: status_port unset\n", path);
 	if(!get_string(c[OPT_SSL_PEER_CN]))
 	{
@@ -715,20 +730,16 @@ static int client_conf_checks(struct conf **c, const char *path, int *r)
 	  && strstr(autoupgrade_os, ".."))
 		conf_problem(path,
 			"autoupgrade_os must not contain a '..' component", r);
-	if(!get_string(c[OPT_CA_BURP_CA]))
+	if(get_string(c[OPT_CA_BURP_CA]))
 	{
 		if(!get_string(c[OPT_CA_CSR_DIR]))
-			conf_problem(path,
-				"ca_burp_ca set, but ca_csr_dir not set\n", r);
+			burp_ca_conf_problem(path, "ca_csr_dir", r);
 		if(!get_string(c[OPT_SSL_CERT_CA]))
-			conf_problem(path,
-				"ca_burp_ca set, but ssl_cert_ca not set\n", r);
+			burp_ca_conf_problem(path, "ssl_cert_ca", r);
 		if(!get_string(c[OPT_SSL_CERT]))
-			conf_problem(path,
-				"ca_burp_ca set, but ssl_cert not set\n", r);
+			burp_ca_conf_problem(path, "ssl_cert", r);
 		if(!get_string(c[OPT_SSL_KEY]))
-			conf_problem(path,
-				"ca_burp_ca set, but ssl_key not set\n", r);
+			burp_ca_conf_problem(path, "ssl_key", r);
 	}
 
 	if(!r)
@@ -781,9 +792,11 @@ static int incexc_munge(struct conf **c, struct strlist *s)
 #ifdef HAVE_WIN32
 	convert_backslashes(&s->path);
 #endif
-	if(path_checks(s->path,
-		"ERROR: Please use absolute include/exclude paths.\n"))
-			return -1;
+	if(!is_absolute(s->path))
+	{
+		logp("ERROR: Please use absolute include/exclude paths.\n");
+		return -1;
+	}
 	if(add_to_strlist(c[OPT_INCEXCDIR], s->path, s->flag))
 		return -1;
 	return 0;
@@ -823,9 +836,11 @@ static int finalise_start_dirs(struct conf **c)
 #ifdef HAVE_WIN32
 		convert_backslashes(&s->path);
 #endif
-		if(path_checks(s->path,
-			"ERROR: Please use absolute include/exclude paths.\n"))
-				return -1;
+		if(!is_absolute(s->path))
+		{
+			logp("ERROR: Please use absolute include/exclude paths.\n");
+			return -1;
+		}
 
 		// Ensure that we do not backup the same directory twice.
 		if(last_ie && !strcmp(s->path, last_ie->path))
@@ -913,11 +928,11 @@ static void set_max_ext(struct strlist *list)
 	if(list) list->flag=max+1;
 }
 
-static int finalise_fstypes(struct conf **c)
+static int finalise_fstypes(struct conf **c, int opt)
 {
 	struct strlist *l;
 	// Set the strlist flag for the excluded fstypes
-	for(l=get_strlist(c[OPT_EXCFS]); l; l=l->next)
+	for(l=get_strlist(c[opt]); l; l=l->next)
 	{
 		l->flag=0;
 		if(!strncasecmp(l->path, "0x", 2))
@@ -955,13 +970,80 @@ static int setup_script_arg_overrides(struct conf *c,
 	  || setup_script_arg_override(c, post_args);
 }
 
-#ifndef UTEST
-static
-#endif
-int conf_finalise(struct conf **c)
+static int finalise_server_ports(struct conf **c,
+	enum conf_opt port_opt, enum conf_opt max_children_opt)
 {
+	struct strlist *p;
+	struct strlist *mc;
+	long max_children=5;
+
+	for(p=get_strlist(c[port_opt]),
+	   mc=get_strlist(c[max_children_opt]); p; p=p->next)
+	{
+		if(mc)
+		{
+			if((max_children=atol(mc->path))<=0)
+			{
+				logp("%s too low for %s %s\n",
+					c[max_children_opt]->field,
+					c[port_opt]->field,
+					p->path);
+				return -1;
+			}
+			p->flag=max_children;
+	
+			mc=mc->next;
+		}
+		else
+		{
+			logp("%s %s defaulting to %s %lu\n",
+				c[port_opt]->field,
+				p->path,
+				c[max_children_opt]->field,
+				max_children);
+			p->flag=max_children;
+		}
+	}
+
+	if(mc)
+	{
+		logp("too many %s options\n", c[max_children_opt]->field);
+		return -1;
+	}
+
+	return 0;
+}
+
+static int finalise_client_ports(struct conf **c)
+{
+	int port=0;
+	struct strlist *p;
+
+	for(p=get_strlist(c[OPT_PORT]); p; p=p->next)
+		port=atoi(p->path);
+	if(!port)
+		return 0;
+
+	if(!get_int(c[OPT_PORT_BACKUP]))
+		set_int(c[OPT_PORT_BACKUP], port);
+	if(!get_int(c[OPT_PORT_RESTORE]))
+		set_int(c[OPT_PORT_RESTORE], port);
+	if(!get_int(c[OPT_PORT_VERIFY]))
+		set_int(c[OPT_PORT_VERIFY], get_int(c[OPT_PORT_RESTORE]));
+	if(!get_int(c[OPT_PORT_LIST]))
+		set_int(c[OPT_PORT_LIST], port);
+	if(!get_int(c[OPT_PORT_DELETE]))
+		set_int(c[OPT_PORT_DELETE], port);
+
+	return 0;
+}
+
+static int conf_finalise(struct conf **c)
+{
+	enum burp_mode burp_mode=get_e_burp_mode(c[OPT_BURP_MODE]);
 	int s_script_notify=0;
-	if(finalise_fstypes(c)) return -1;
+	if(finalise_fstypes(c, OPT_EXCFS)) return -1;
+	if(finalise_fstypes(c, OPT_INCFS)) return -1;
 
 	strlist_compile_regexes(get_strlist(c[OPT_INCREG]));
 	strlist_compile_regexes(get_strlist(c[OPT_EXCREG]));
@@ -970,7 +1052,7 @@ int conf_finalise(struct conf **c)
 	set_max_ext(get_strlist(c[OPT_EXCEXT]));
 	set_max_ext(get_strlist(c[OPT_EXCOM]));
 
-	if(get_e_burp_mode(c[OPT_BURP_MODE])==BURP_MODE_CLIENT
+	if(burp_mode==BURP_MODE_CLIENT
 	  && finalise_glob(c)) return -1;
 
 	if(finalise_incexc_dirs(c)
@@ -978,6 +1060,20 @@ int conf_finalise(struct conf **c)
 
 	if(finalise_keep_args(c)) return -1;
 
+	if(burp_mode==BURP_MODE_SERVER)
+	{
+		if(finalise_server_ports(c,
+			OPT_PORT, OPT_MAX_CHILDREN)
+		  || finalise_server_ports(c,
+			OPT_STATUS_PORT, OPT_MAX_STATUS_CHILDREN))
+				return -1;
+	}
+	if(burp_mode==BURP_MODE_CLIENT)
+	{
+		if(finalise_client_ports(c))
+			return -1;
+	}
+
 	if((s_script_notify=get_int(c[OPT_S_SCRIPT_NOTIFY])))
 	{
 		set_int(c[OPT_S_SCRIPT_PRE_NOTIFY], s_script_notify);
@@ -1012,9 +1108,6 @@ static int conf_finalise_global_only(const char *conf_path, struct conf **confs)
 {
 	int r=0;
 
-	if(!get_string(confs[OPT_PORT]))
-		conf_problem(conf_path, "port unset", &r);
-
 	// Let the caller check the 'keep' value.
 
 	if(!get_string(confs[OPT_SSL_KEY_PASSWORD])
@@ -1314,12 +1407,16 @@ static int restore_client_allowed(struct conf **cconfs, struct conf **sconfs)
 	return 0;
 }
 
-// FIX THIS: need to unit test this.
 int conf_switch_to_orig_client(struct conf **globalcs,
 	struct conf **cconfs, const char *orig_client)
 {
 	int ret=-1;
 	struct conf **sconfs=NULL;
+
+	// If we are already the wanted client, no need to switch.
+	if(!strcmp(get_string(cconfs[OPT_CNAME]), orig_client))
+		return 0;
+
 	if(!(sconfs=confs_alloc())
 	  || confs_init(sconfs)) goto end;
 	if(set_string(sconfs[OPT_CNAME], orig_client))
@@ -1361,3 +1458,30 @@ end:
 	confs_free(&sconfs);
 	return ret;
 }
+
+char *config_default_path(void)
+{
+	static char path[256]="";
+#ifdef HAVE_WIN32
+	char *pfenv=NULL;
+
+	// Burp used to always install to 'C:/Program Files/Burp/', but as
+	// of 1.3.11, it changed to %PROGRAMFILES%. Still want the old way
+	// to work though. So check %PROGRAMFILES% first, then fall back.
+	if((pfenv=getenv("PROGRAMFILES")))
+	{
+		struct stat statp;
+		snprintf(path, sizeof(path), "%s/%s/%s.conf",
+			pfenv, PACKAGE_NAME, PACKAGE_TARNAME);
+		if(!lstat(path, &statp)
+		  && !S_ISDIR(statp.st_mode))
+			return path;
+	}
+	snprintf(path, sizeof(path), "C:/Program Files/%s/%s.conf",
+		PACKAGE_NAME, PACKAGE_TARNAME);
+#else
+	snprintf(path, sizeof(path), "%s/%s.conf",
+		SYSCONFDIR, PACKAGE_TARNAME);
+#endif
+	return path;
+}
