@@ -4,7 +4,7 @@
 #include "cntr.h"
 #include "strlist.h"
 
-extern int conf_get_pair(char buf[], char **field, char **value);
+extern int conf_get_pair(char buf[], char **field, char **value, int *reset);
 extern int conf_parse_incexcs_buf(struct conf **confs, const char *incexc);
 extern int conf_parse_incexcs_path(struct conf **confs, const char *path);
 
@@ -19,9 +19,10 @@ extern int conf_switch_to_orig_client(struct conf **globalcs,
 
 extern int reeval_glob(struct conf **c);
 
+extern char *config_default_path(void);
+
 #ifdef UTEST
 extern int conf_load_lines_from_buf(const char *buf, struct conf **c);
-extern int conf_finalise(struct conf **c);
 extern int conf_load_overrides(struct conf **globalcs, struct conf **cconfs,
 	const char *path);
 #endif
