@@ -30,10 +30,12 @@ extern int entries_in_directory_alphasort(const char *path,
 	char ***nl, int *count, int atime);
 extern int filter_dot(const struct dirent *d);
 
+extern int files_equal(const char *opath, const char *npath, int compressed);
+
 #ifndef HAVE_WIN32
 extern int mksock(const char *path);
 
-extern int is_lnk(const char *path);
+extern int is_lnk_lstat(const char *path);
 extern int is_lnk_valid(const char *path);
 extern int do_symlink(const char *oldpath, const char *newpath);
 extern int readlink_w(const char *path, char buf[], size_t buflen);
