@@ -25,9 +25,6 @@ extern int set_keepalive(int fd, int value);
 extern int init_client_socket(const char *host, const char *port);
 extern void reuseaddr(int fd);
 extern int chuser_and_or_chgrp(const char *user, const char *group);
-extern const char *getdatestr(time_t t);
-extern const char *getlocaldatestr(time_t t);
-extern const char *time_taken(time_t d);
 extern int dpth_protocol1_is_compressed(int compressed, const char *datapath);
 #ifndef HAVE_WIN32
 extern void setup_signal(int sig, void handler(int sig));
@@ -35,7 +32,7 @@ extern void setup_signal(int sig, void handler(int sig));
 
 extern long version_to_long(const char *version);
 
-/* These receive_a_file() and send_file() functions are for use by extra_comms
+/* These receive_a_file() and send_a_file() functions are for use by extra_comms
    and the CA stuff, rather than backups/restores. */
 extern int receive_a_file(struct asfd *asfd,
 	const char *path, struct cntr *cntr);
@@ -55,8 +52,6 @@ extern void free_w(char **str);
 extern char *strlwr(char *s);
 extern void strip_fqdn(char **fqdn);
 
-extern int astrcat(char **buf, const char *append, const char *func);
-
 extern void strip_trailing_slashes(char **str);
 
 extern int breakpoint(int breaking, const char *func);
@@ -65,6 +60,4 @@ extern int breakpoint(int breaking, const char *func);
 extern void convert_backslashes(char **path);
 #endif
 
-extern char *encode_time(time_t utime, char *buf);
-
 #endif
