@@ -7,6 +7,7 @@
 #include "iobuf.h"
 #include "log.h"
 #include "strlist.h"
+#include "times.h"
 
 const char *prog="unknown";
 const char *prog_long="unknown";
@@ -17,6 +18,7 @@ static struct fzp *logfzp=NULL;
 static int do_syslog=1;
 static int do_stdout=1;
 static int do_progress_counter=1;
+static int force_quiet=0;
 static int syslog_opened=0;
 static int json=0;
 
@@ -27,22 +29,6 @@ void log_init(char *progname)
 	else prog=progname;
 }
 
-#ifndef UTEST
-static char *gettm(void)
-{
-        time_t t=0;
-        const struct tm *ctm=NULL;
-        static char tmbuf[32]="";
-
-        time(&t);
-        ctm=localtime(&t);
-	// Windows does not like the %T strftime format option - you get
-	// complaints under gdb.
-        strftime(tmbuf, sizeof(tmbuf), "%Y-%m-%d %H:%M:%S", ctm);
-	return tmbuf;
-}
-#endif
-
 void logp(const char *fmt, ...)
 {
 #ifndef UTEST
@@ -53,7 +39,8 @@ void logp(const char *fmt, ...)
 	vsnprintf(buf, sizeof(buf), fmt, ap);
 	pid=(int)getpid();
 	if(logfzp)
-		fzp_printf(logfzp, "%s: %s[%d] %s", gettm(), prog, pid, buf);
+		fzp_printf(logfzp, "%s: %s[%d] %s",
+			gettimenow(), prog, pid, buf);
 	else
 	{
 		if(do_syslog)
@@ -62,15 +49,23 @@ void logp(const char *fmt, ...)
 		{
 			if(json)
 			{
-				char *cp=NULL;
-				if((cp=strrchr(buf, '\n'))) *cp='\0';
+				char *cp;
 				// To help programs parsing the monitor output,
 				// log things with simple JSON.
+				// So do simple character substitution to have
+				// a better chance of valid JSON.
+				for(cp=buf; *cp; cp++)
+				{
+					if(*cp=='"')
+						*cp='\'';
+					else if(!isprint(*cp))
+						*cp='.';
+				}
 				fprintf(stdout, "{ \"logline\": \"%s\" }\n", buf);
 			}
 			else
 				fprintf(stdout, "%s: %s[%d] %s",
-					gettm(), prog, pid, buf);
+					gettimenow(), prog, pid, buf);
 		}
 	}
 	va_end(ap);
@@ -129,11 +124,14 @@ void logc(const char *fmt, ...)
 void logfmt(const char *fmt, ...)
 {
 #ifndef UTEST
-	char buf[512]="";
-	va_list ap;
-	va_start(ap, fmt);
-	vsnprintf(buf, sizeof(buf), fmt, ap);
-	fprintf(stdout, "%s", buf);
+	if(do_stdout)
+	{
+		char buf[512]="";
+		va_list ap;
+		va_start(ap, fmt);
+		vsnprintf(buf, sizeof(buf), fmt, ap);
+		fprintf(stdout, "%s", buf);
+	}
 #endif
 }
 
@@ -152,8 +150,16 @@ int log_fzp_set(const char *path, struct conf **confs)
 	}
 	if(logfzp) fzp_setlinebuf(logfzp);
 	do_syslog=get_int(confs[OPT_SYSLOG]);
-	do_stdout=get_int(confs[OPT_STDOUT]);
-	do_progress_counter=get_int(confs[OPT_PROGRESS_COUNTER]);
+	if(force_quiet)
+	{
+		do_stdout=0;
+		do_progress_counter=0;
+	}
+	else
+	{
+		do_stdout=get_int(confs[OPT_STDOUT]);
+		do_progress_counter=get_int(confs[OPT_PROGRESS_COUNTER]);
+	}
 
 	if(syslog_opened)
 	{
@@ -168,6 +174,11 @@ int log_fzp_set(const char *path, struct conf **confs)
 	return 0;
 }
 
+void log_force_quiet(void)
+{
+	force_quiet=1;
+}
+
 void log_fzp_set_direct(struct fzp *fzp)
 {
 	fzp_close(&logfzp);
@@ -199,6 +210,9 @@ void log_restore_settings(struct conf **cconfs, int srestore)
 	if(get_string(cconfs[OPT_RESTOREPREFIX]))
 		logp("restoreprefix = '%s'\n",
 			get_string(cconfs[OPT_RESTOREPREFIX]));
+	if(get_string(cconfs[OPT_STRIP_FROM_PATH]))
+		logp("stripfrompath = '%s'\n",
+			get_string(cconfs[OPT_STRIP_FROM_PATH]));
 	if(get_string(cconfs[OPT_REGEX]))
 		logp("regex = '%s'\n", get_string(cconfs[OPT_REGEX]));
 	for(l=get_strlist(cconfs[OPT_INCLUDE]); l; l=l->next)
@@ -294,6 +308,7 @@ int log_incexcs_buf(const char *incexc)
 
 void log_recvd(struct iobuf *iobuf, struct cntr *cntr, int print)
 {
+	int newline=0;
 	const char *prefix="unset";
 	switch(iobuf->cmd)
 	{
@@ -301,6 +316,8 @@ void log_recvd(struct iobuf *iobuf, struct cntr *cntr, int print)
 		case CMD_WARNING: prefix="WARNING"; break;
 		default: break;
 	}
-	logp("%s: %s", prefix, iobuf->buf);
+	if(iobuf->buf[iobuf->len]!='\n')
+		newline=1;
+	logp("%s: %s%s", prefix, iobuf->buf, newline?"\n":"");
 	cntr_add(cntr, iobuf->cmd, print);
 }
