@@ -11,48 +11,25 @@
 #include "server/main.h"
 #include "server/protocol1/bedup.h"
 #include "server/protocol2/bsigs.h"
+#include "server/protocol2/bsparse.h"
 #include "server/protocol2/champ_chooser/champ_server.h"
 
-static char *get_conf_path(void)
-{
-	static char path[256]="";
-#ifdef HAVE_WIN32
-	char *pfenv=NULL;
-
-	// Burp used to always install to 'C:/Program Files/Burp/', but as
-	// of 1.3.11, it changed to %PROGRAMFILES%. Still want the old way
-	// to work though. So check %PROGRAMFILES% first, then fall back.
-	if((pfenv=getenv("PROGRAMFILES")))
-	{
-		struct stat statp;
-		snprintf(path, sizeof(path), "%s/Burp/burp.conf", pfenv);
-		if(!lstat(path, &statp)
-		  && !S_ISDIR(statp.st_mode))
-			return path;
-	}
-	snprintf(path, sizeof(path), "C:/Program Files/Burp/burp.conf");
-#else
-	snprintf(path, sizeof(path), "%s", SYSCONFDIR "/burp.conf");
-#endif
-	return path;
-}
-
 static void usage_server(void)
 {
 #ifndef HAVE_WIN32
-	printf("\nThe configuration file specifies whether burp runs in server or client mode.\n");
+	printf("\nThe configuration file specifies whether %s runs in server or client mode.\n", PACKAGE_TARNAME);
 	printf("\nServer usage: %s [options]\n", progname());
 	printf("\n");
 	printf(" Options:\n");
 	printf("  -a c          Run as a stand-alone champion chooser.\n");
-	printf("  -c <path>     Path to conf file (default: %s).\n", get_conf_path());
+	printf("  -c <path>     Path to conf file (default: %s).\n", config_default_path());
 	printf("  -d <path>     a single client in the status monitor.\n");
 	printf("  -F            Stay in the foreground.\n");
 	printf("  -g            Generate initial CA certificates and exit.\n");
 	printf("  -h|-?         Print this text and exit.\n");
 	printf("  -i            Print index of symbols and exit.\n");
-	printf("  -l <path>     Log file for the status monitor.\n");
 	printf("  -n            Do not fork any children (implies '-F').\n");
+	printf("  -Q            Do not log to stdout (overrides config file)\n");
 	printf("  -t            Dry-run to test config file syntax.\n");
 	printf("  -v            Print version and exit.\n");
 	printf("Options to use with '-a c':\n");
@@ -83,30 +60,36 @@ static void usage_client(void)
 	printf("                  T: check backup timer, but do not actually backup\n");
 	printf("                  v: verify\n");
 	printf("  -b <number>    Backup number (default: the most recent backup).\n");
-	printf("  -c <path>      Path to conf file (default: %s).\n", get_conf_path());
+	printf("  -c <path>      Path to conf file (default: %s).\n", config_default_path());
 	printf("  -d <directory> Directory to restore to, or directory to list.\n");
 	printf("  -f             Allow overwrite during restore.\n");
 	printf("  -h|-?          Print this text and exit.\n");
 	printf("  -i             Print index of symbols and exit.\n");
 	printf("  -q <max secs>  Randomised delay of starting a timed backup.\n");
+	printf("  -Q             Do not log to stdout (overrides config file)\n");
 	printf("  -r <regex>     Specify a regular expression.\n");
 	printf("  -s <number>    Number of leading path components to strip during restore.\n");
 	printf("  -t             Dry-run to test config file syntax.\n");
 	printf("  -v             Print version and exit.\n");
-#ifndef HAVE_WIN32
+#ifdef HAVE_WIN32
 	printf("  -x             Do not use the Windows VSS API when restoring.\n");
+#else
+	printf("  -x             Strip Windows VSS data when restoring.\n");
 	printf("Options to use with '-a S':\n");
 	printf("  -C <client>   Show a particular client.\n");
 	printf("  -b <number>   Show listable files in a particular backup (requires -C).\n");
-	printf("  -z <file>     Dump a particular log file in a backup (requires -C and -b).\n");
 	printf("  -d <path>     Show a particular path in a backup (requires -C and -b).\n");
+	printf("  -l <path>     Log file for the status monitor.\n");
+	printf("  -z <file>     Dump a particular log file in a backup (requires -C and -b).\n");
 #endif
 	printf("\n");
 #ifndef HAVE_WIN32
-	printf(" See http://burp.grke.net/ or the man page ('man burp') for usage examples\n");
+	printf(" See %s or the man page ('man %s') for usage examples\n",
+		PACKAGE_URL, PACKAGE_TARNAME);
 	printf(" and additional configuration options.\n\n");
 #else
-	printf(" See http://burp.grke.net/ for usage examples and additional configuration\n");
+	printf(" See %s for usage examples and additional configuration\n",
+		PACKAGE_TARNAME);
 	printf(" options.\n\n");
 #endif
 }
@@ -130,11 +113,6 @@ int reload(struct conf **confs, const char *conffile, bool firsttime)
 		setup_signals();
 #endif
 
-	// Do not try to change user or group after the first time.
-	if(firsttime && chuser_and_or_chgrp(
-		get_string(confs[OPT_USER]), get_string(confs[OPT_GROUP])))
-			return -1;
-
 	return 0;
 }
 
@@ -244,7 +222,7 @@ static int run_test_confs(struct conf **confs, const char *client)
 		goto end;
 	confs_init(cconfs);
 	if(set_string(cconfs[OPT_CNAME], client)
-	  || set_string(cconfs[OPT_PEER_VERSION], VERSION)
+	  || set_string(cconfs[OPT_PEER_VERSION], PACKAGE_VERSION)
 	  || conf_load_clientconfdir(confs, cconfs))
 		goto end;
 	confs_dump(cconfs, CONF_FLAG_CC_OVERRIDE|CONF_FLAG_INCEXC);
@@ -254,8 +232,34 @@ end:
 	return ret;
 }
 
-#if defined(HAVE_WIN32)
-#define main BurpMain
+static struct lock *get_prog_lock(struct conf **confs)
+{
+	struct lock *lock=NULL;
+	const char *lockfile=confs_get_lockfile(confs);
+	if(!(lock=lock_alloc_and_init(lockfile)))
+		goto error;
+	lock_get(lock);
+	switch(lock->status)
+	{
+		case GET_LOCK_GOT:
+			return lock;
+		case GET_LOCK_NOT_GOT:
+			logp("Could not get lockfile.\n");
+			logp("Another process is probably running.\n");
+			goto error;
+		case GET_LOCK_ERROR:
+		default:
+			logp("Could not get lockfile.\n");
+			logp("Maybe you do not have permissions to write to %s.\n", lockfile);
+			goto error;
+	}
+error:
+	lock_free(&lock);
+	return NULL;
+}
+
+#ifdef HAVE_WIN32
+#define main RealMain
 #endif
 #ifndef UTEST
 static
@@ -275,10 +279,11 @@ int real_main(int argc, char *argv[])
 	const char *backup=NULL;
 	const char *backup2=NULL;
 	char *restoreprefix=NULL;
+	char *stripfrompath=NULL;
 	const char *regex=NULL;
 	const char *browsefile=NULL;
 	char *browsedir=NULL;
-	const char *conffile=get_conf_path();
+	const char *conffile=config_default_path();
 	const char *orig_client=NULL;
 	const char *logfile=NULL;
 	// The orig_client is the original client that the normal client
@@ -296,9 +301,11 @@ int real_main(int argc, char *argv[])
 		return run_bedup(argc, argv);
 	if(!strcmp(prog, "bsigs"))
 		return run_bsigs(argc, argv);
+	if(!strcmp(prog, "bsparse"))
+		return run_bsparse(argc, argv);
 #endif
 
-	while((option=getopt(argc, argv, "a:b:c:C:d:fFghil:nq:r:s:tvxjz:?"))!=-1)
+	while((option=getopt(argc, argv, "a:b:c:C:d:fFghil:nq:Qr:s:tvxjz:?"))!=-1)
 	{
 		switch(option)
 		{
@@ -345,6 +352,9 @@ int real_main(int argc, char *argv[])
 			case 'q':
 				randomise=atoi(optarg);
 				break;
+			case 'Q':
+				log_force_quiet();
+				break;
 			case 'r':
 				regex=optarg;
 				break;
@@ -352,7 +362,7 @@ int real_main(int argc, char *argv[])
 				strip=atoi(optarg);
 				break;
 			case 'v':
-				printf("%s-%s\n", progname(), VERSION);
+				printf("%s-%s\n", progname(), PACKAGE_VERSION);
 				ret=0;
 				goto end;
 			case 'x':
@@ -381,11 +391,9 @@ int real_main(int argc, char *argv[])
 	{
 		// Try to output everything in JSON.
 		log_set_json(1);
-#ifndef HAVE_WIN32
 		// Need to do this so that processes reading stdout get the
 		// result of the printfs of logp straight away.
-		setlinebuf(stdout);
-#endif
+		setvbuf(stdout, NULL, _IONBF, 0);
 	}
 
 	if(!(confs=confs_alloc()))
@@ -444,42 +452,39 @@ int real_main(int argc, char *argv[])
 	  && (act==ACTION_BACKUP_TIMED || act==ACTION_TIMER_CHECK))
 		random_delay(confs);
 
-	if(mode==BURP_MODE_SERVER
-	  && act==ACTION_CHAMP_CHOOSER)
+	if(mode==BURP_MODE_SERVER)
 	{
-		// These server modes need to run without getting the lock.
-	}
-	else if(mode==BURP_MODE_CLIENT
-	  && (act==ACTION_LIST
-		|| act==ACTION_LIST_LONG
-		|| act==ACTION_DIFF
-		|| act==ACTION_DIFF_LONG
-		|| act==ACTION_STATUS
-		|| act==ACTION_STATUS_SNAPSHOT
-		|| act==ACTION_MONITOR))
-	{
-		// These client modes need to run without getting the lock.
-	}
-	else
-	{
-		const char *lockfile=confs_get_lockfile(confs);
-		if(!(lock=lock_alloc_and_init(lockfile)))
-			goto end;
-		lock_get(lock);
-		switch(lock->status)
+		switch(act)
 		{
-			case GET_LOCK_GOT: break;
-			case GET_LOCK_NOT_GOT:
-				logp("Could not get lockfile.\n");
-				logp("Another process is probably running,\n");
-				goto end;
-			case GET_LOCK_ERROR:
+			case ACTION_CHAMP_CHOOSER:
+				// Need to run without getting the lock.
+				break;
 			default:
-				logp("Could not get lockfile.\n");
-				logp("Maybe you do not have permissions to write to %s.\n", lockfile);
-				goto end;
+				if(!(lock=get_prog_lock(confs)))
+					goto end;
+				break;
 		}
 	}
+	else if(mode==BURP_MODE_CLIENT)
+	{
+		switch(act)
+		{
+			case ACTION_BACKUP:
+			case ACTION_BACKUP_TIMED:
+			case ACTION_TIMER_CHECK:
+				// Need to get the lock.
+				if(!(lock=get_prog_lock(confs)))
+					goto end;
+				break;
+			default:
+				break;
+		}
+	}
+
+	// Change privileges after having got the lock, for convenience.
+	if(chuser_and_or_chgrp(
+		get_string(confs[OPT_USER]), get_string(confs[OPT_GROUP])))
+			return -1;
 
 	set_int(confs[OPT_OVERWRITE], forceoverwrite);
 	set_int(confs[OPT_STRIP], strip);
@@ -491,6 +496,7 @@ int real_main(int argc, char *argv[])
 	if(replace_conf_str(confs[OPT_BACKUP], backup)
 	  || replace_conf_str(confs[OPT_BACKUP2], backup2)
 	  || replace_conf_str(confs[OPT_RESTOREPREFIX], restoreprefix)
+	  || replace_conf_str(confs[OPT_STRIP_FROM_PATH], stripfrompath)
 	  || replace_conf_str(confs[OPT_REGEX], regex)
 	  || replace_conf_str(confs[OPT_BROWSEFILE], browsefile)
 	  || replace_conf_str(confs[OPT_BROWSEDIR], browsedir)
