@@ -11,12 +11,20 @@
        __typeof__ (b) _b = (b); \
      _a < _b ? _a : _b; })
 
-extern int send_whole_file_gzl(struct asfd *asfd, const char *datapth,
+enum send_e
+{
+	SEND_FATAL=-1,
+	SEND_OK=0,
+	SEND_ERROR=1
+};
+
+extern enum send_e send_whole_file_gzl(struct asfd *asfd, const char *datapth,
 	int quick_read, uint64_t *bytes, const char *encpassword,
 	struct cntr *cntr, int compression, struct BFILE *bfd,
-	const char *extrameta, size_t elen);
+	const char *extrameta, size_t elen,
+	int key_deriv, uint64_t salt);
 
-extern int send_whole_filel(struct asfd *asfd,
+extern enum send_e send_whole_filel(struct asfd *asfd,
 #ifdef HAVE_WIN32
 	enum cmd cmd,
 #endif
@@ -25,7 +33,8 @@ extern int send_whole_filel(struct asfd *asfd,
 	struct BFILE *bfd,
 	const char *extrameta, size_t elen);
 
-extern EVP_CIPHER_CTX *enc_setup(int encrypt, const char *encryption_password);
+extern EVP_CIPHER_CTX *enc_setup(int encrypt, const char *encryption_password,
+	int key_deriv, uint64_t salt);
 
 extern char *get_endfile_str(uint64_t bytes, uint8_t *checksum);
 extern int write_endfile(struct asfd *asfd, uint64_t bytes, uint8_t *checksum);
