@@ -5,6 +5,7 @@
 #include "../../cmd.h"
 #include "../../cntr.h"
 #include "../../conf.h"
+#include "../../cstat.h"
 #include "../../fsops.h"
 #include "../../fzp.h"
 #include "../../handy.h"
@@ -353,6 +354,7 @@ static int jiggle(struct sdirs *sdirs, struct fdirs *fdirs, struct sbuf *sb,
 	char *newpath=NULL;
 	char *finpath=NULL;
 	char *deltafpath=NULL;
+	char *relinkpath=NULL;
 	const char *datapth=sb->protocol1->datapth.buf;
 
 	// If the previous backup was a hardlinked_archive, there will not be
@@ -450,12 +452,24 @@ static int jiggle(struct sdirs *sdirs, struct fdirs *fdirs, struct sbuf *sb,
 		goto end;
 	}
 
+	if(!(relinkpath=prepend_s(sdirs->relink, datapth)))
+		goto end;
+	if(!lstat(relinkpath, &statp) && S_ISREG(statp.st_mode))
+	{
+		// Use the relinked path - something that used to be a hardlink
+		// but is now the original file because the file that we
+		// originally hardlinked to has been deleted.
+		ret=do_rename(relinkpath, finpath);
+		goto end;
+	}
+
 	logp("could not find: %s\n", oldpath);
 end:
 	free_w(&oldpath);
 	free_w(&newpath);
 	free_w(&finpath);
 	free_w(&deltafpath);
+	free_w(&relinkpath);
 	return ret;
 }
 
@@ -525,14 +539,14 @@ static int maybe_delete_files_from_manifest(const char *manifesttmp,
 	while(omzp || dfp)
 	{
 		if(dfp && !db->path.buf
-		  && (ars=sbuf_fill_from_file(db, dfp, NULL, NULL)))
+		  && (ars=sbuf_fill_from_file(db, dfp, NULL)))
 		{
 			if(ars<0) goto end;
 			// ars==1 means it ended ok.
 			fzp_close(&dfp);
 		}
 		if(omzp && !mb->path.buf
-		  && (ars=sbuf_fill_from_file(mb, omzp, NULL, NULL)))
+		  && (ars=sbuf_fill_from_file(mb, omzp, NULL)))
 		{
 			if(ars<0) goto end;
 			// ars==1 means it ended ok.
@@ -643,7 +657,7 @@ static int atomic_data_jiggle(struct sdirs *sdirs, struct fdirs *fdirs,
 
 	while(1)
 	{
-		switch(sbuf_fill_from_file(sb, zp, NULL, NULL))
+		switch(sbuf_fill_from_file(sb, zp, NULL))
 		{
 			case 0: break;
 			case 1: goto end;
@@ -809,6 +823,7 @@ int backup_phase4_server_protocol1(struct sdirs *sdirs, struct conf **cconfs)
 	// This will have the effect of getting rid of unnecessary
 	// directories.
 	recursive_delete_dirs_only_no_warnings(fdirs->currentdupdata);
+	recursive_delete_dirs_only_no_warnings(sdirs->relink);
 
 	// Rename the old current to something that we know to delete.
 	if(previous_backup && !hardlinked_current)
