@@ -29,6 +29,9 @@ static struct lock *locklist=NULL;
 
 static int verbose=0;
 
+static unsigned int maxlinks=DEF_MAX_LINKS;
+static char ext[16]="";
+
 typedef struct file file_t;
 
 struct file
@@ -261,7 +264,7 @@ static int get_full_cksum(struct file *f, struct fzp **fzp)
 }
 
 /* Make it atomic by linking to a temporary file, then moving it into place. */
-static int do_hardlink(struct file *o, struct file *n, const char *ext)
+static int do_hardlink(struct file *o, struct file *n)
 {
 	int ret=-1;
 	char *tmppath=NULL;
@@ -277,7 +280,15 @@ static int do_hardlink(struct file *o, struct file *n, const char *ext)
 		goto end;
 	}
 	if((ret=do_rename(tmppath, o->path)))
+	{
+		// 'man 2 rename', says it should be safe to unlink tmppath:
+		// "If newpath exists but the operation fails for some reason,
+		// rename() guarantees to leave an instance of newpath in
+		// place."
+		if(unlink(tmppath))
+			logp("Could not unlink %s\n", tmppath);
 		goto end;
+	}
 	ret=0;
 end:
 	free_w(&tmppath);
@@ -289,14 +300,17 @@ static void reset_old_file(struct file *oldfile, struct file *newfile,
 {
 	//printf("reset %s with %s %d\n", oldfile->path, newfile->path,
 	//	info->st_nlink);
-	oldfile->nlink=info->st_nlink;
+	struct file *next;
+
+	next=oldfile->next;
 	free_w(&oldfile->path);
-	oldfile->path=newfile->path;
+	memcpy(oldfile, newfile, sizeof(struct file));
+	oldfile->next=next;
 	newfile->path=NULL;
 }
 
 static int check_files(struct mystruct *find, struct file *newfile,
-	struct stat *info, const char *ext, unsigned int maxlinks)
+	struct stat *info)
 {
 	int found=0;
 	struct fzp *nfp=NULL;
@@ -326,6 +340,12 @@ static int check_files(struct mystruct *find, struct file *newfile,
 			found++;
 			break;
 		}
+		if(newfile->nlink>=maxlinks) {
+			// This new file file has enough links. Just leave it
+			// as it is to avoid undoing all these hardlinks.
+			found++;
+			break;
+		}
 		if((!newfile->part_cksum && get_part_cksum(newfile, &nfp))
 		  || (!f->part_cksum && get_part_cksum(f, &ofp)))
 		{
@@ -382,33 +402,16 @@ static int check_files(struct mystruct *find, struct file *newfile,
 		// Now hardlink it.
 		if(makelinks)
 		{
-			switch(do_hardlink(newfile, f, ext))
+			if(do_hardlink(newfile, f))
 			{
-				case 0:
-					f->nlink++;
-					// Only count bytes as saved if we
-					// removed the last link.
-					if(newfile->nlink==1)
-						savedbytes+=info->st_size;
-					break;
-				case -1:
-					// On error, replace the memory of the
-					// old file with the one that we just
-					// found. It might work better when
-					// someone later tries to link to the
-					// new one instead of the old one.
-					reset_old_file(f, newfile, info);
-					count--;
-					break;
-				default:
-					// Abandon all hope.
-					// This could happen if renaming the
-					// hardlink failed in such a way that
-					// the target file was unlinked without
-					// being replaced - ie, if the max
-					// number of hardlinks is being hit.
-					return -1;
+				count--;
+				return -1;
 			}
+			f->nlink++;
+			// Only count bytes as saved if we
+			// removed the last link.
+			if(newfile->nlink==1)
+				savedbytes+=info->st_size;
 		}
 		else if(deletedups)
 		{
@@ -459,7 +462,7 @@ static int looks_like_protocol1(const char *basedir)
 	}
 	// If there is a 'current' symlink here, we think it looks like a
 	// protocol 1 backup.
-	if(is_lnk(tmp)>0)
+	if(is_lnk_lstat(tmp)>0)
 	{
 		ret=1;
 		goto end;
@@ -514,7 +517,7 @@ static int level_exclusion(int level, const char *fname,
 
 // Return 0 for directory processed, -1 for error, 1 for not processed.
 static int process_dir(const char *oldpath, const char *newpath,
-	const char *ext, unsigned int maxlinks, int burp_mode, int level)
+	int burp_mode, int level)
 {
 	int ret=-1;
 	DIR *dirp=NULL;
@@ -571,7 +574,8 @@ static int process_dir(const char *oldpath, const char *newpath,
 
 		if(S_ISDIR(info.st_mode))
 		{
-			if(process_dir(path, dirinfo->d_name, ext, maxlinks,					burp_mode, level+1))
+			if(process_dir(path, dirinfo->d_name,
+				burp_mode, level+1))
 					goto end;
 			continue;
 		}
@@ -589,7 +593,7 @@ static int process_dir(const char *oldpath, const char *newpath,
 		if((find=find_key(info.st_size)))
 		{
 			//printf("check %d: %s\n", info.st_size, newfile.path);
-			if(check_files(find, &newfile, &info, ext, maxlinks))
+			if(check_files(find, &newfile, &info))
 				goto end;
 		}
 		else
@@ -601,7 +605,7 @@ static int process_dir(const char *oldpath, const char *newpath,
 	}
 	ret=0;
 end:
-	closedir(dirp);
+	if(dirp) closedir(dirp);
 	free_w(&newfile.path);
 	free_w(&path);
 	return ret;
@@ -639,7 +643,7 @@ static int in_group(struct strlist *grouplist, const char *dedup_group)
 }
 
 static int iterate_over_clients(struct conf **globalcs,
-	struct strlist *grouplist, const char *ext, unsigned int maxlinks)
+	struct strlist *grouplist)
 {
 	int ret=0;
 	DIR *dirp=NULL;
@@ -647,10 +651,6 @@ static int iterate_over_clients(struct conf **globalcs,
 	struct dirent *dirinfo=NULL;
 	const char *globalclientconfdir=get_string(globalcs[OPT_CLIENTCONFDIR]);
 
-	signal(SIGABRT, &sighandler);
-	signal(SIGTERM, &sighandler);
-	signal(SIGINT, &sighandler);
-
 	if(!(cconfs=confs_alloc())) return -1;
 	if(confs_init(cconfs)) return -1;
 
@@ -728,7 +728,7 @@ static int iterate_over_clients(struct conf **globalcs,
 
 		switch(process_dir(get_string(cconfs[OPT_DIRECTORY]),
 			dirinfo->d_name,
-			ext, maxlinks, 1 /* burp mode */, 0 /* level */))
+			1 /* burp mode */, 0 /* level */))
 		{
 			case 0: ccount++;
 			case 1: continue;
@@ -745,11 +745,102 @@ static int iterate_over_clients(struct conf **globalcs,
 	return ret;
 }
 
-static char *get_config_path(void)
+static int process_from_conf(const char *configfile, char **groups)
 {
-        static char path[256]="";
-        snprintf(path, sizeof(path), "%s", SYSCONFDIR "/burp.conf");
-        return path;
+	int ret=-1;
+	struct conf **globalcs=NULL;
+	struct strlist *grouplist=NULL;
+	struct lock *globallock=NULL;
+
+	signal(SIGABRT, &sighandler);
+	signal(SIGTERM, &sighandler);
+	signal(SIGINT, &sighandler);
+
+	if(*groups)
+	{
+		char *tok=NULL;
+		if((tok=strtok(*groups, ",\n")))
+		{
+			do
+			{
+				if(strlist_add(&grouplist, tok, 1))
+				{
+					log_out_of_memory(__func__);
+					goto end;
+				}
+			} while((tok=strtok(NULL, ",\n")));
+		}
+		if(!grouplist)
+		{
+			logp("unable to read list of groups\n");
+			goto end;
+		}
+	}
+
+	// Read directories from config files, and get locks.
+	if(!(globalcs=confs_alloc())
+	  || confs_init(globalcs)
+	  || conf_load_global_only(configfile, globalcs))
+		goto end;
+
+	if(get_e_burp_mode(globalcs[OPT_BURP_MODE])!=BURP_MODE_SERVER)
+	{
+		logp("%s is not a server config file\n", configfile);
+		goto end;
+	}
+	logp("Dedup clients from %s\n",
+		get_string(globalcs[OPT_CLIENTCONFDIR]));
+	maxlinks=get_int(globalcs[OPT_MAX_HARDLINKS]);
+	if(grouplist)
+	{
+		struct strlist *g=NULL;
+		logp("in dedup groups:\n");
+		for(g=grouplist; g; g=g->next)
+			logp("%s\n", g->path);
+	}
+	else
+	{
+		char *lockpath=NULL;
+		const char *opt_lockfile=confs_get_lockfile(globalcs);
+		// Only get the global lock when doing a global run.
+		// If you are doing individual groups, you are likely
+		// to want to do many different dedup jobs and a
+		// global lock would get in the way.
+		if(!(lockpath=prepend(opt_lockfile, ".bedup"))
+		  || !(globallock=lock_alloc_and_init(lockpath)))
+			goto end;
+		lock_get(globallock);
+		if(globallock->status!=GET_LOCK_GOT)
+		{
+			logp("Could not get lock %s (%d)\n", lockpath,
+				globallock->status);
+			free_w(&lockpath);
+			goto end;
+		}
+		logp("Got %s\n", lockpath);
+	}
+	ret=iterate_over_clients(globalcs, grouplist);
+end:
+	confs_free(&globalcs);
+	lock_release(globallock);
+	lock_free(&globallock);
+	strlists_free(&grouplist);
+	return ret;
+}
+
+static int process_from_command_line(int argc, char *argv[])
+{
+	int i;
+	for(i=optind; i<argc; i++)
+	{
+		// Strip trailing slashes, for tidiness.
+		if(argv[i][strlen(argv[i])-1]=='/')
+			argv[i][strlen(argv[i])-1]='\0';
+		if(process_dir("", argv[i],
+			0 /* not burp mode */, 0 /* level */))
+				return 1;
+	}
+	return  0;
 }
 
 static int usage(void)
@@ -757,7 +848,7 @@ static int usage(void)
 	logfmt("\nUsage: %s [options]\n", prog);
 	logfmt("\n");
 	logfmt(" Options:\n");
-	logfmt("  -c <path>                Path to config file (default: %s).\n", get_config_path());
+	logfmt("  -c <path>                Path to config file (default: %s).\n", config_default_path());
 	logfmt("  -g <list of group names> Only run on the directories of clients that\n");
 	logfmt("                           are in one of the groups specified.\n");
 	logfmt("                           The list is comma-separated. To put a client in a\n");
@@ -765,40 +856,37 @@ static int usage(void)
 	logfmt("                           configuration file on the server.\n");
 	logfmt("  -h|-?                    Print this text and exit.\n");
 	logfmt("  -d                       Delete any duplicate files found.\n");
-	logfmt("                           (non-burp mode only)\n");
+	logfmt("                           (non-%s mode only)\n", PACKAGE_TARNAME);
 	logfmt("  -l                       Hard link any duplicate files found.\n");
 	logfmt("  -m <number>              Maximum number of hard links to a single file.\n");
-	logfmt("                           (non-burp mode only - in burp mode, use the\n");
+	logfmt("                           (non-%s mode only - in burp mode, use the\n", PACKAGE_TARNAME);
 	logfmt("                           max_hardlinks option in the configuration file)\n");
 	logfmt("                           The default is %d. On ext3, the maximum number\n", DEF_MAX_LINKS);
 	logfmt("                           of links possible is 32000, but space is needed\n");
-	logfmt("                           for the normal operation of burp.\n");
-	logfmt("  -n <list of directories> Non-burp mode. Deduplicate any (set of) directories.\n");
+	logfmt("                           for the normal operation of %s.\n", PACKAGE_TARNAME);
+	logfmt("  -n <list of directories> Non-%s mode. Deduplicate any (set of) directories.\n", PACKAGE_TARNAME);
 	logfmt("  -v                       Print duplicate paths.\n");
 	logfmt("  -V                       Print version and exit.\n");
 	logfmt("\n");
-	logfmt("By default, %s will read %s and deduplicate client storage\n", prog, get_config_path());
+	logfmt("By default, %s will read %s and deduplicate client storage\n", prog, config_default_path());
 	logfmt("directories using special knowledge of the structure.\n");
 	logfmt("\n");
 	logfmt("With '-n', this knowledge is turned off and you have to specify the directories\n");
 	logfmt("to deduplicate on the command line. Running with '-n' is therefore dangerous\n");
-	logfmt("if you are deduplicating burp storage directories.\n\n");
+	logfmt("if you are deduplicating %s storage directories.\n\n", PACKAGE_TARNAME);
 	return 1;
 }
 
 int run_bedup(int argc, char *argv[])
 {
-	int i=1;
 	int ret=0;
 	int option=0;
 	int nonburp=0;
-	unsigned int maxlinks=DEF_MAX_LINKS;
 	char *groups=NULL;
-	char ext[16]="";
 	int givenconfigfile=0;
 	const char *configfile=NULL;
 
-	configfile=get_config_path();
+	configfile=config_default_path();
 	snprintf(ext, sizeof(ext), ".bedup.%d", getpid());
 
 	while((option=getopt(argc, argv, "c:dg:hlm:nvV?"))!=-1)
@@ -825,7 +913,7 @@ int run_bedup(int argc, char *argv[])
 				nonburp=1;
 				break;
 			case 'V':
-				logfmt("%s-%s\n", prog, VERSION);
+				logfmt("%s-%s\n", prog, PACKAGE_VERSION);
 				return 0;
 			case 'v':
 				verbose=1;
@@ -848,7 +936,7 @@ int run_bedup(int argc, char *argv[])
 	}
 	if(!nonburp && maxlinks!=DEF_MAX_LINKS)
 	{
-		logp("-m option is specified via the configuration file in burp mode (max_hardlinks=)\n");
+		logp("-m option is specified via the configuration file in %s mode (max_hardlinks=)\n", PACKAGE_TARNAME);
 		return 1;
 	}
 	if(deletedups && makelinks)
@@ -888,93 +976,13 @@ int run_bedup(int argc, char *argv[])
 	if(nonburp)
 	{
 		// Read directories from command line.
-		for(i=optind; i<argc; i++)
-		{
-			// Strip trailing slashes, for tidiness.
-			if(argv[i][strlen(argv[i])-1]=='/')
-				argv[i][strlen(argv[i])-1]='\0';
-			if(process_dir("", argv[i], ext, maxlinks,
-				0 /* not burp mode */, 0 /* level */))
-			{
-				ret=1;
-				break;
-			}
-		}
+		if(process_from_command_line(argc, argv))
+			ret=1;
 	}
 	else
 	{
-		struct conf **globalcs=NULL;
-		struct strlist *grouplist=NULL;
-		struct lock *globallock=NULL;
-
-		if(groups)
-		{
-			char *tok=NULL;
-			if((tok=strtok(groups, ",\n")))
-			{
-				do
-				{
-					if(strlist_add(&grouplist, tok, 1))
-					{
-						log_out_of_memory(__func__);
-						return -1;
-					}
-				} while((tok=strtok(NULL, ",\n")));
-			}
-			if(!grouplist)
-			{
-				logp("unable to read list of groups\n");
-				return -1;
-			}
-		}
-
-		// Read directories from config files, and get locks.
-		if(!(globalcs=confs_alloc())) return -1;
-		if(confs_init(globalcs)) return -1;
-		if(conf_load_global_only(configfile, globalcs)) return 1;
-		if(get_e_burp_mode(globalcs[OPT_BURP_MODE])!=BURP_MODE_SERVER)
-		{
-			logp("%s is not a server config file\n", configfile);
-			confs_free(&globalcs);
-			return 1;
-		}
-		logp("Dedup clients from %s\n",
-			get_string(globalcs[OPT_CLIENTCONFDIR]));
-		maxlinks=get_int(globalcs[OPT_MAX_HARDLINKS]);
-		if(grouplist)
-		{
-			struct strlist *g=NULL;
-			logp("in dedup groups:\n");
-			for(g=grouplist; g; g=g->next)
-				logp("%s\n", g->path);
-		}
-		else
-		{
-			char *lockpath=NULL;
-			const char *opt_lockfile=confs_get_lockfile(globalcs);
-			// Only get the global lock when doing a global run.
-			// If you are doing individual groups, you are likely
-			// to want to do many different dedup jobs and a
-			// global lock would get in the way.
-			if(!(lockpath=prepend(opt_lockfile, ".bedup"))
-			  || !(globallock=lock_alloc_and_init(lockpath)))
-				return 1;
-			lock_get(globallock);
-			if(globallock->status!=GET_LOCK_GOT)
-			{
-				logp("Could not get lock %s (%d)\n", lockpath,
-					globallock->status);
-				free_w(&lockpath);
-				return 1;
-			}
-			logp("Got %s\n", lockpath);
-		}
-		ret=iterate_over_clients(globalcs, grouplist, ext, maxlinks);
-		confs_free(&globalcs);
-
-		lock_release(globallock);
-		lock_free(&globallock);
-		strlists_free(&grouplist);
+		if(process_from_conf(configfile, &groups))
+			ret=1;
 	}
 
 	if(!nonburp)
