@@ -6,6 +6,7 @@
 #include "../../base64.h"
 #include "../../cmd.h"
 #include "../../cntr.h"
+#include "../../cstat.h"
 #include "../../handy.h"
 #include "../../hexmap.h"
 #include "../../iobuf.h"
@@ -14,6 +15,7 @@
 #include "../../protocol2/blist.h"
 #include "../../protocol2/rabin/rabin.h"
 #include "../../slist.h"
+#include "../child.h"
 #include "../manios.h"
 #include "../resume.h"
 #include "champ_chooser/champ_client.h"
@@ -27,6 +29,10 @@
 #define END_REQUESTS		0x04
 #define END_BLK_REQUESTS	0x08
 
+#define BLKS_MAX_UNREQUESTED	BLKS_MAX_IN_MEM/4
+// This needs to be greater than BLKS_MAX_UNREQUESTED
+#define SLIST_MAX_IN_MEM	BLKS_MAX_IN_MEM
+
 static int breaking=0;
 static int breakcount=0;
 
@@ -126,7 +132,7 @@ static int entry_changed(struct sbuf *sb,
 		return 1;
 	}
 
-	if((*csb)->path.buf)
+	if(*csb && (*csb)->path.buf)
 	{
 		// Already have an entry.
 	}
@@ -318,7 +324,6 @@ static int deal_with_read(struct iobuf *rbuf, struct slist *slist,
 		case CMD_MESSAGE:
 		case CMD_WARNING:
 		{
-			struct cntr *cntr=NULL;
 			log_recvd(rbuf, cntr, 0);
 			goto end;
 		}
@@ -488,6 +493,7 @@ static int sbuf_needs_data(struct sbuf *sb, struct asfd *asfd,
 	struct blk *blk;
 	static struct iobuf wbuf;
 	struct blist *blist=slist->blist;
+	static int unrequested=0;
 
 	if(!(sb->flags & SBUF_HEADER_WRITTEN_TO_MANIFEST))
 	{
@@ -524,20 +530,22 @@ static int sbuf_needs_data(struct sbuf *sb, struct asfd *asfd,
 				// adjacent blocks are deduplicated well.
 				if(hash_load_blk(blk))
 					goto end;
-
-				if(!blk->requested)
-				{
-					// Also let the client know, so that it
-					// can free memory if there was a long
-					// consecutive number of unrequested
-					// blocks.
-					get_wbuf_from_index(&wbuf, blk->index);
-					if(asfd->write(asfd, &wbuf))
-						goto end;
-				}
 			}
 		}
 
+		if(!blk->requested)
+			unrequested++;
+
+		if(unrequested>BLKS_MAX_UNREQUESTED)
+		{
+			unrequested=0;
+			// Let the client know that it can free memory if there
+			// was a long consecutive number of unrequested blocks.
+			get_wbuf_from_index(&wbuf, blk->index);
+			if(asfd->write(asfd, &wbuf))
+				goto end;
+		}
+
 		if(blk==sb->protocol2->bend)
 		{
 			blist_adjust_head(blist, sb);
@@ -608,12 +616,8 @@ static int maybe_add_from_scan(struct manios *manios,
 		sbuf_free(&snew);
 		if(!manios->phase1) return 0;
 		// Limit the amount loaded into memory at any one time.
-		if(slist && slist->head)
-		{
-			if(slist->head->protocol2->index
-			  - slist->tail->protocol2->index>4096)
-				return 0;
-		}
+		if(slist->count>SLIST_MAX_IN_MEM)
+			return 0;
 		if(!(snew=sbuf_alloc(PROTO_2))) goto end;
 
 		switch(manio_read(manios->phase1, snew))
@@ -833,8 +837,8 @@ static int check_for_missing_work_in_slist(struct slist *slist)
 	{
 		if(!(sb->flags & SBUF_END_WRITTEN_TO_MANIFEST))
 		{
-			logp("ERROR: finishing but still waiting for: %c:%s\n",
-				slist->head->path.cmd, slist->head->path.buf);
+			logp("ERROR: finishing but still waiting for: %s\n",
+				iobuf_to_printable(&slist->head->path));
 			return -1;
 		}
 	}
@@ -910,7 +914,7 @@ int do_backup_phase2_server_protocol2(struct async *as, struct asfd *chfd,
 	{
 		if(!(p1pos=do_resume(sdirs, dpth, confs)))
 			goto end;
-		if(cntr_send_sdirs(asfd, sdirs, confs))
+		if(cntr_send_sdirs(asfd, sdirs, confs, CNTR_STATUS_BACKUP))
 			goto end;
 	}
 
@@ -924,6 +928,10 @@ int do_backup_phase2_server_protocol2(struct async *as, struct asfd *chfd,
 	memset(&wbuf, 0, sizeof(struct iobuf));
 	while(!(end_flags&END_BACKUP))
 	{
+		if(write_status(CNTR_STATUS_BACKUP,
+			csb && csb->path.buf?csb->path.buf:"", cntr))
+				goto end;
+
 		if(maybe_add_from_scan(manios, slist, chfd, &csb, cntr))
 			goto end;
 
