@@ -5,8 +5,11 @@
 #include "../bu.h"
 #include "../cmd.h"
 #include "../cntr.h"
+#include "../cstat.h"
 #include "../handy.h"
+#include "../hexmap.h"
 #include "../linkhash.h"
+#include "../lock.h"
 #include "../log.h"
 #include "../pathcmp.h"
 #include "../prepend.h"
@@ -23,6 +26,7 @@
 #include "protocol2/rblk.h"
 #include "protocol2/restore.h"
 #include "../protocol2/rabin/rabin.h"
+#include "rubble.h"
 #include "sdirs.h"
 
 static enum asl_ret restore_end_func(struct asfd *asfd,
@@ -69,15 +73,26 @@ static int check_srestore(struct conf **confs, const char *path)
 	return 0;
 }
 
-int want_to_restore(int srestore, struct sbuf *sb,
-	regex_t *regex, struct conf **cconfs)
+static int want_to_restore(int srestore, struct sbuf *sb,
+	regex_t *regex, enum action act , struct conf **cconfs)
 {
+	if(act==ACTION_RESTORE
+	  && !get_int(cconfs[OPT_CLIENT_IS_WINDOWS]))
+	{
+		// Do not send VSS data to non-windows.
+		if(sbuf_is_vssdata(sb))
+			return 0;
+		// Do not send VSS directory data to non-windows.
+		if(S_ISDIR(sb->statp.st_mode) && sbuf_is_filedata(sb))
+			return 0;
+	}
 	return (!srestore || check_srestore(cconfs, sb->path.buf))
 	  && (!regex || regex_check(regex, sb->path.buf));
 }
 
 static int setup_cntr(struct asfd *asfd, const char *manifest,
-        regex_t *regex, int srestore, struct conf **cconfs)
+	regex_t *regex, int srestore, struct conf **cconfs, enum action act,
+	struct bu *bu)
 {
 	int ars=0;
 	int ret=-1;
@@ -87,6 +102,7 @@ static int setup_cntr(struct asfd *asfd, const char *manifest,
 
 	cntr=get_cntr(cconfs);
 	if(!cntr) return 0;
+	cntr->bno=(int)bu->bno;
 
 // FIX THIS: this is only trying to work for protocol1.
 	if(get_protocol(cconfs)!=PROTO_1) return 0;
@@ -99,7 +115,7 @@ static int setup_cntr(struct asfd *asfd, const char *manifest,
 	}
 	while(1)
 	{
-		if((ars=sbuf_fill_from_file(sb, fzp, NULL, NULL)))
+		if((ars=sbuf_fill_from_file(sb, fzp, NULL)))
 		{
 			if(ars<0) goto end;
 			// ars==1 means end ok
@@ -107,7 +123,7 @@ static int setup_cntr(struct asfd *asfd, const char *manifest,
 		}
 		else
 		{
-			if(want_to_restore(srestore, sb, regex, cconfs))
+			if(want_to_restore(srestore, sb, regex, act, cconfs))
 			{
 				cntr_add_phase1(cntr, sb->path.cmd, 0);
 				if(sb->endfile.buf)
@@ -131,6 +147,16 @@ static int restore_sbuf(struct asfd *asfd, struct sbuf *sb, struct bu *bu,
 	struct conf **cconfs, struct sbuf *need_data, const char *manifest,
 	struct slist *slist);
 
+static void log_missing_block(struct asfd *asfd, struct cntr *cntr,
+	struct blk *blk, struct sbuf *need_data)
+{
+	uint16_t datno=0;
+	char *savepathstr;
+	savepathstr=uint64_to_savepathstr_with_sig_uint(blk->savepath, &datno);
+	logw(asfd, cntr, "%s: Missing block %s:%d\n",
+		iobuf_to_printable(&need_data->path), savepathstr, datno);
+}
+
 // Used when restoring a hard link that we have not restored the destination
 // for. Read through the manifest from the beginning and substitute the path
 // and data to the new location.
@@ -158,13 +184,15 @@ static int hard_link_substitution(struct asfd *asfd,
 	if(protocol==PROTO_2)
 	{
 		  if(!(blk=blk_alloc()))
-                	goto end;
+			goto end;
 	}
 
 	while(1)
 	{
+		if(blk)
+			blk->got_save_path=0;
 		switch(manio_read_with_blk(manio,
-			hb, need_data->path.buf?blk:NULL, sdirs))
+			hb, need_data->path.buf?blk:NULL))
 		{
 			case 0: break; // Keep going.
 			case 1: ret=0; goto end; // Finished OK.
@@ -178,6 +206,17 @@ static int hard_link_substitution(struct asfd *asfd,
 				sbuf_free_content(hb);
 				continue;
 			}
+			if(blk->got_save_path)
+			{
+				blk->got_save_path=0;
+				if(rblk_retrieve_data(asfd, cntr,
+					blk, sdirs->data))
+				{
+					log_missing_block(asfd, cntr,
+						blk, need_data);
+					continue;
+				}
+			}
 			if(blk->data)
 			{
 				if(protocol2_extra_restore_stream_bits(asfd,
@@ -223,7 +262,8 @@ static int restore_sbuf(struct asfd *asfd, struct sbuf *sb, struct bu *bu,
 	struct conf **cconfs, struct sbuf *need_data, const char *manifest,
 	struct slist *slist)
 {
-	//printf("%s: %s\n", act==ACTION_RESTORE?"restore":"verify", sb->path.buf);
+	//printf("%s: %s\n", act==ACTION_RESTORE?"restore":"verify",
+	//  iobuf_to_printable(&sb->path));
 	if(write_status(cntr_status, sb->path.buf, get_cntr(cconfs)))
 		return -1;
 
@@ -297,6 +337,14 @@ int restore_ent(struct asfd *asfd,
 			  act, sdirs, cntr_status, cconfs, need_data, manifest,
 			  slist))
 				goto end;
+			if(get_protocol(cconfs)==PROTO_2
+			  && sbuf_is_filedata(xb)
+			  && get_int(cconfs[OPT_CLIENT_IS_WINDOWS]))
+			{
+				// Windows directories need endfile to be sent.
+				if(asfd->write(asfd, &xb->endfile))
+					goto end;
+			}
 			slist->head=xb->next;
 			sbuf_free(&xb);
 		}
@@ -355,8 +403,15 @@ static int restore_remaining_dirs(struct asfd *asfd, struct bu *bu,
 		else
 		{
 			if(restore_sbuf_protocol2(asfd, sb, act,
-				get_cntr(cconfs), need_data))
+				get_cntr(cconfs), NULL))
 					goto end;
+			if(sbuf_is_filedata(sb)
+			  && get_int(cconfs[OPT_CLIENT_IS_WINDOWS]))
+			{
+				// Windows directories need endfile to be sent.
+				if(asfd->write(asfd, &sb->endfile))
+					goto end;
+			}
 		}
 	}
 	ret=0;
@@ -366,12 +421,13 @@ end:
 }
 
 static int restore_stream(struct asfd *asfd, struct sdirs *sdirs,
-        struct slist *slist, struct bu *bu, const char *manifest,
+	struct slist *slist, struct bu *bu, const char *manifest,
 	regex_t *regex, int srestore, struct conf **cconfs, enum action act,
-        enum cntr_status cntr_status)
+	enum cntr_status cntr_status)
 {
 	int ret=-1;
 	int last_ent_was_dir=0;
+	int last_ent_was_skipped=0;
 	struct sbuf *sb=NULL;
 	struct iobuf *rbuf=asfd->rbuf;
 	struct manio *manio=NULL;
@@ -380,6 +436,7 @@ static int restore_stream(struct asfd *asfd, struct sdirs *sdirs,
 	enum protocol protocol=get_protocol(cconfs);
 	struct cntr *cntr=get_cntr(cconfs);
 	struct iobuf interrupt;
+
 	iobuf_init(&interrupt);
 
 	if(protocol==PROTO_2)
@@ -436,8 +493,10 @@ static int restore_stream(struct asfd *asfd, struct sdirs *sdirs,
 				goto end;
 		}
 
+		if(blk)
+			blk->got_save_path=0;
 		switch(manio_read_with_blk(manio,
-			sb, need_data->path.buf?blk:NULL, sdirs))
+			sb, need_data->path.buf?blk:NULL))
 		{
 			case 0: break; // Keep going.
 			case 1: ret=0; goto end; // Finished OK.
@@ -448,9 +507,25 @@ static int restore_stream(struct asfd *asfd, struct sdirs *sdirs,
 		{
 			if(sb->endfile.buf)
 			{
-				if(act==ACTION_RESTORE
-				  && asfd->write(asfd, &sb->endfile))
-					goto end;
+				if(act==ACTION_RESTORE && !last_ent_was_skipped)
+				{
+					if(last_ent_was_dir)
+					{
+						// Delay sending endfile until
+						// we actually send the
+						// directory.
+						struct sbuf *xb=slist->head;
+						iobuf_free_content(&xb->endfile);
+						iobuf_move(&xb->endfile,
+							&sb->endfile);
+					}
+					else
+					{
+						if(asfd->write(asfd,
+							&sb->endfile))
+								goto end;
+					}
+				}
 				sbuf_free_content(sb);
 				iobuf_free_content(&interrupt);
 				continue;
@@ -464,8 +539,17 @@ static int restore_stream(struct asfd *asfd, struct sdirs *sdirs,
 				else if(!iobuf_pathcmp(&need_data->path,
 					&interrupt))
 				{
-					if(blk->data)
-						blk->data=NULL;
+					continue;
+				}
+			}
+			if(blk->got_save_path)
+			{
+				blk->got_save_path=0;
+				if(rblk_retrieve_data(asfd, cntr,
+					blk, sdirs->data))
+				{
+					log_missing_block(asfd, cntr,
+						blk, need_data);
 					continue;
 				}
 			}
@@ -479,21 +563,26 @@ static int restore_stream(struct asfd *asfd, struct sdirs *sdirs,
 			sbuf_free_content(need_data);
 		}
 
-		if(want_to_restore(srestore, sb, regex, cconfs))
+		if(want_to_restore(srestore, sb, regex, act, cconfs))
 		{
+			last_ent_was_skipped=0;
 			if(restore_ent(asfd, &sb, slist,
 				bu, act, sdirs, cntr_status, cconfs,
 				need_data, &last_ent_was_dir, manifest))
 					goto end;
 		}
-		else if(sbuf_is_filedata(sb) || sbuf_is_vssdata(sb))
+		else
 		{
-			// Add it to the list of filedata that was not
-			// restored.
-			struct f_link **bucket=NULL;
-			if(!linkhash_search(&sb->statp, &bucket)
-			  && linkhash_add(sb->path.buf, &sb->statp, bucket))
-				goto end;
+			last_ent_was_skipped=1;
+			if(sbuf_is_filedata(sb) || sbuf_is_vssdata(sb))
+			{
+				// Add it to the list of filedata that was not
+				// restored.
+				struct f_link **bucket=NULL;
+				if(!linkhash_search(&sb->statp, &bucket)
+				  && linkhash_add(sb->path.buf, &sb->statp, bucket))
+					goto end;
+			}
 		}
 
 		sbuf_free_content(sb);
@@ -512,20 +601,19 @@ static int actual_restore(struct asfd *asfd, struct bu *bu,
 	const char *manifest, regex_t *regex, int srestore, enum action act,
 	struct sdirs *sdirs, enum cntr_status cntr_status, struct conf **cconfs)
 {
-        int ret=-1;
-        // For out-of-sequence directory restoring so that the
-        // timestamps come out right:
-        struct slist *slist=NULL;
+	int ret=-1;
+	// For out-of-sequence directory restoring so that the
+	// timestamps come out right:
+	struct slist *slist=NULL;
 	struct cntr *cntr=NULL;
 
 	if(linkhash_init()
-          || !(slist=slist_alloc()))
-                goto end;
-
-	if(get_protocol(cconfs)==PROTO_2
-	  && rblk_init())
+	  || !(slist=slist_alloc()))
 		goto end;
 
+	if(get_protocol(cconfs)==PROTO_2)
+		rblks_init(get_uint64_t(cconfs[OPT_RBLK_MEMORY_MAX]));
+
 	if(restore_stream(asfd, sdirs, slist,
 		bu, manifest, regex,
 		srestore, cconfs, act, cntr_status))
@@ -538,10 +626,10 @@ static int actual_restore(struct asfd *asfd, struct bu *bu,
 	cntr_print(cntr, act, asfd);
 	cntr_stats_to_file(cntr, bu->path, act);
 end:
-        slist_free(&slist);
+	slist_free(&slist);
 	linkhash_free();
-	rblk_free();
-        return ret;
+	rblks_free();
+	return ret;
 }
 
 static int get_logpaths(struct bu *bu, const char *file,
@@ -553,6 +641,34 @@ static int get_logpaths(struct bu *bu, const char *file,
 	return 0;
 }
 
+static void parallelism_warnings(struct asfd *asfd, struct conf **cconfs,
+	struct sdirs *sdirs, struct bu *bu, enum protocol protocol)
+{
+	struct bu *b;
+
+	if(lock_test(sdirs->lock_storage_for_write->path))
+	{
+		logm(asfd, cconfs, "Another process is currently backing up or deleting for this client.\n");
+		return;
+	}
+
+	if(!check_for_rubble(sdirs))
+		return;
+
+	for(b=bu; b && b->next; b=b->next)
+	{
+		if(b->flags & BU_CURRENT)
+			break; // Warning.
+		if(protocol==PROTO_2)
+			return; // No warning.
+		if(b->flags & BU_HARDLINKED)
+			return; // No warning.
+	}
+
+	logw(asfd, get_cntr(cconfs),
+		"The latest backup needs recovery, but continuing anyway.\n");
+}
+
 static int restore_manifest(struct asfd *asfd, struct bu *bu,
 	regex_t *regex, int srestore, enum action act, struct sdirs *sdirs,
 	char **dir_for_notify, struct conf **cconfs)
@@ -563,14 +679,27 @@ static int restore_manifest(struct asfd *asfd, struct bu *bu,
 	char *logpathz=NULL;
 	enum protocol protocol;
 	enum cntr_status cntr_status;
+	struct lock *lock=NULL;
+	char *lockfile=NULL;
 	static int manifest_count=0;
 
 	protocol=get_protocol(cconfs);
-
 	if(protocol==PROTO_2
 	  && blks_generate_init())
 		goto end;
 
+	if(!(lockfile=prepend_s(bu->path, "lockfile.read"))
+	  || !(lock=lock_alloc_and_init(lockfile)))
+		goto end;
+	lock_get(lock);
+	if(lock->status!=GET_LOCK_GOT)
+	{
+		char msg[256]="";
+		snprintf(msg, sizeof(msg), "Another process is restoring or verifying backup %s.\n", bu->timestamp);
+		log_and_send(asfd, msg);
+		goto end;
+	}
+
 	// For sending status information up to the server.
 	cntr_status=CNTR_STATUS_RESTORING;
 
@@ -606,7 +735,7 @@ static int restore_manifest(struct asfd *asfd, struct bu *bu,
 	// This is the equivalent of a phase1 scan during backup.
 
 	if(setup_cntr(asfd, manifest,
-		regex, srestore, cconfs))
+		regex, srestore, cconfs, act, bu))
 			goto end;
 
 	if(!manifest_count)
@@ -617,21 +746,28 @@ static int restore_manifest(struct asfd *asfd, struct bu *bu,
 		// This means that the client side 'expected' counter will be
 		// confusing in that case. Live with it for now.
 		// However, the server side log will be OK.
-		if(cntr_send_bu(asfd, bu, cconfs))
+		if(cntr_send_bu(asfd, bu, cconfs, cntr_status))
 			goto end;
 	}
 
+	parallelism_warnings(asfd, cconfs, sdirs, bu, protocol);
+
 	// Now, do the actual restore.
 	ret=actual_restore(asfd, bu, manifest,
 		  regex, srestore, act, sdirs, cntr_status, cconfs);
 end:
 	log_fzp_set(NULL, cconfs);
-	compress_file(logpath, logpathz, get_int(cconfs[OPT_COMPRESSION]));
+	if(logpath && logpathz)
+		compress_file(logpath, logpathz,
+			get_int(cconfs[OPT_COMPRESSION]));
 	free_w(&manifest);
 	free_w(&logpath);
 	free_w(&logpathz);
 	if(protocol==PROTO_2)
 		blks_generate_free();
+	free_w(&lockfile);
+	lock_release(lock);
+	lock_free(&lock);
 	manifest_count++;
 	return ret;
 }
