@@ -41,8 +41,6 @@
 #include "mem_pool.h"
 #include "berrno.h"
 
-#define MAX_PATHLENGTH  1024
-
 /* UTF-8 to UCS2 path conversion is expensive,
    so we cache the conversion. During backup the
    conversion is called 3 times (lstat, attribs, open),
@@ -1206,6 +1204,7 @@ err:
 
 int closedir(DIR *dirp)
 {
+	if(!dirp) return 0;
 	_dir *dp=(_dir *)dirp;
 	FindClose(dp->dirh);
 	_dir_free(dp);
@@ -1215,7 +1214,7 @@ int closedir(DIR *dirp)
 static void copyin(struct dirent *entry, const char *fname)
 {
 	char *cp=entry->d_name;
-	while(*fname)
+	while( *fname && entry->d_reclen < (MAX_PATH_UTF8-1) )
 	{
 		*cp++=*fname++;
 		entry->d_reclen++;
@@ -1312,16 +1311,16 @@ long pathconf(const char *path, int name)
 	{
 		case _PC_PATH_MAX:
 			if(!strncmp(path, "\\\\?\\", 4))
-				return 32767;
+				return MAX_PATH_W;
 		case _PC_NAME_MAX:
-			return 255;
+			return MAX_PATH;
 	}
 	return system_error();
 }
 
 int WSA_Init(void)
 {
-	WORD wVersionRequested=MAKEWORD( 1, 1);
+	WORD wVersionRequested=MAKEWORD(1, 1);
 	WSADATA wsaData;
 
 	if(WSAStartup(wVersionRequested, &wsaData))
@@ -1345,7 +1344,7 @@ static int win32_chmod_old(const char *path, mode_t mode)
 		attr=p_GetFileAttributesW((LPCWSTR)pwszBuf);
 		if(attr!=INVALID_FILE_ATTRIBUTES)
 		{
-			// Use Burp mappings defined in stat() above.
+			// Use mappings defined in stat() above.
 			if(!(mode & (S_IRUSR|S_IRGRP|S_IROTH)))
 				attr |= FILE_ATTRIBUTE_READONLY;
 			else
@@ -1783,7 +1782,7 @@ const char * getArgv0(const char *cmdline)
 
 static int dwAltNameLength_ok(DWORD dwAltNameLength)
 {
-	return dwAltNameLength>0 && dwAltNameLength<MAX_PATHLENGTH;
+	return dwAltNameLength>0 && dwAltNameLength<MAX_PATH_UTF8;
 }
 
 /* Extracts the executable or script name from the first string in 
@@ -1883,11 +1882,11 @@ bool GetApplicationName(const char *cmdline, char **pexe, const char **pargs)
 	DWORD dwBasePathLength=pExeEnd-pExeStart;
 
 	DWORD dwAltNameLength=0;
-	char *pPathname=(char *)alloca(MAX_PATHLENGTH+1);
-	char *pAltPathname=(char *)alloca(MAX_PATHLENGTH+1);
+	char *pPathname=(char *)alloca(MAX_PATH_UTF8+1);
+	char *pAltPathname=(char *)alloca(MAX_PATH_UTF8+1);
 
-	pPathname[MAX_PATHLENGTH]='\0';
-	pAltPathname[MAX_PATHLENGTH]='\0';
+	pPathname[MAX_PATH_UTF8]='\0';
+	pAltPathname[MAX_PATH_UTF8]='\0';
 
 	memcpy(pPathname, pExeStart, dwBasePathLength);
 	pPathname[dwBasePathLength]='\0';
@@ -1904,7 +1903,7 @@ bool GetApplicationName(const char *cmdline, char **pexe, const char **pargs)
 				// There are no path separators, search in the
 				// standard locations
 				dwAltNameLength=SearchPath(NULL, pPathname,
-				  ExtensionList[index], MAX_PATHLENGTH,
+				  ExtensionList[index], MAX_PATH_UTF8,
 				  pAltPathname, NULL);
 				if(dwAltNameLength_ok(dwAltNameLength))
 				{
@@ -1917,7 +1916,7 @@ bool GetApplicationName(const char *cmdline, char **pexe, const char **pargs)
 			else
 			{
 				snprintf(&pPathname[dwBasePathLength],
-					MAX_PATHLENGTH-dwBasePathLength,
+					MAX_PATH_UTF8-dwBasePathLength,
 					"%s", ExtensionList[index]);
 				if(GetFileAttributes(pPathname)
 				  !=INVALID_FILE_ATTRIBUTES)
@@ -1931,7 +1930,7 @@ bool GetApplicationName(const char *cmdline, char **pexe, const char **pargs)
 		// There are no path separators, search in the standard
 		// locations.
 		dwAltNameLength=SearchPath(NULL, pPathname,
-			NULL, MAX_PATHLENGTH, pAltPathname, NULL);
+			NULL, MAX_PATH_UTF8, pAltPathname, NULL);
 		if(dwAltNameLength_ok(dwAltNameLength))
 		{
 			memcpy(pPathname, pAltPathname, dwAltNameLength);
@@ -1942,7 +1941,7 @@ bool GetApplicationName(const char *cmdline, char **pexe, const char **pargs)
 	if(strchr(pPathname, ' '))
 	{
 		dwAltNameLength=GetShortPathName(pPathname,
-			pAltPathname, MAX_PATHLENGTH);
+			pAltPathname, MAX_PATH_UTF8);
 
 		if(dwAltNameLength_ok(dwAltNameLength))
 		{
@@ -2016,16 +2015,19 @@ pid_t forkchild_no_wait(struct fzp **sin, struct fzp **sout, struct fzp **serr,
 	return do_forkchild(sin, sout, serr, path, argv, 0 /* do not wait */);
 }
 
-int win32_utime(const char *fname, struct utimbuf *times)
+int win32_utime(const char *fname, struct stat *statp)
 {
+	FILETIME cre;
 	FILETIME acc;
 	FILETIME mod;
 	char tmpbuf[5000];
 
 	conv_unix_to_win32_path(fname, tmpbuf, 5000);
 
-	cvt_utime_to_ftime(times->actime, acc);
-	cvt_utime_to_ftime(times->modtime, mod);
+	// We save creation date in st_ctime.
+	cvt_utime_to_ftime(statp->st_ctime, cre);
+	cvt_utime_to_ftime(statp->st_atime, acc);
+	cvt_utime_to_ftime(statp->st_mtime, mod);
 
 	HANDLE h=INVALID_HANDLE_VALUE;
 
@@ -2071,8 +2073,61 @@ int win32_utime(const char *fname, struct utimbuf *times)
 		return -1;
 	}
 
-	int rval=SetFileTime(h, NULL, &acc, &mod)?0:-1;
+	int rval=SetFileTime(h, &cre, &acc, &mod)?0:-1;
 	CloseHandle(h);
 	if(rval==-1) errno=b_errno_win32;
 	return rval;
 }
+
+int win32_getfsname(const char *path, char *fsname, size_t fsname_size)
+{
+	// I do not think anyone still needs non-Unicode stuff.
+	WCHAR fsname_ucs2[MAX_PATH + 1];
+	{
+		WCHAR *pwsz_path=(WCHAR*)sm_get_pool_memory(PM_FNAME);
+		make_win32_path_UTF8_2_wchar((char**)&pwsz_path, path);
+		int path_len=wcslen(pwsz_path);
+		if (path_len && pwsz_path[path_len-1] != '\\')
+		{
+			pwsz_path=(WCHAR*)sm_check_pool_memory_size((char*)pwsz_path,
+				sizeof(WCHAR)*path_len+sizeof(L"\\"));
+			wcscpy(pwsz_path+path_len++, L"\\");
+		}
+
+		int error;
+		for(;;)
+		{
+			error=GetVolumeInformationW(
+				pwsz_path /* lpRootPathName */,
+				NULL /* lpVolumeNameBuffer */,
+				0 /* nVolumeNameSize */,
+				NULL /* lpVolumeSerialNumber */,
+				NULL /* lpMaximumComponentLength */,
+				NULL /* lpFileSystemFlags */,
+				fsname_ucs2 /* lpFileSystemNameBuffer */,
+				MAX_PATH + 1 /* nFileSystemNameSize */
+			) ? 0 : GetLastError();
+			if(error!=ERROR_DIR_NOT_ROOT)
+				break;
+
+			// We are not in root, let's try upper directory.
+			do --path_len; while(path_len && pwsz_path[path_len-1]!='\\');
+			if (!path_len)
+				break;
+			// Terminate string right after earlier slash.
+			pwsz_path[path_len]=0;
+		}
+		sm_free_pool_memory((char*)pwsz_path);
+
+		if (error)
+		{
+			char used_path[MAX_PATH_UTF8 + 1];
+			wchar_2_UTF8(used_path, (WCHAR*)pwsz_path, sizeof(used_path));
+			fprintf(stderr, "Cannot get volume information for %s: ERR=%d\n",
+				used_path, error);
+			return error;
+		}
+	}
+	wchar_2_UTF8(fsname, fsname_ucs2, fsname_size);
+	return 0;
+}
