@@ -38,37 +38,40 @@ static void tear_down(struct conf ***globalcs, struct conf ***confs)
 
 struct data
 {
-        const char *str;
+	const char *str;
 	const char *field;
 	const char *value;
+	int reset;
 };
 
 static struct data d[] = {
-	{ "a=b", "a", "b" },
-	{ "a=b\n", "a", "b" },
-	{ "a = b", "a", "b" },
-	{ "   a  =    b ", "a", "b" },
-	{ "   a  =    b \n", "a", "b" },
-	{ "#a=b", NULL, NULL },
-	{ "  #a=b", NULL, NULL },
-	{ "a='b'", "a", "b" },
-	{ "a='b", "a", "b" },
-	{ "a=b'", "a", "b'" },
-	{ "a=\"b\"", "a", "b" },
-	{ "a=b\"", "a", "b\"" },
-	{ "a=\"b", "a", "b" },
-	{ "a=b # comment", "a", "b # comment" }, // Maybe fix this.
-	{ "field=longvalue with spaces", "field", "longvalue with spaces" },
+	{ "a=b", "a", "b", 0 },
+	{ "a=b\n", "a", "b", 0 },
+	{ "a = b", "a", "b", 0 },
+	{ "   a  =    b ", "a", "b", 0 },
+	{ "   a  =    b \n", "a", "b", 0 },
+	{ "#a=b", NULL, NULL, 0 },
+	{ "  #a=b", NULL, NULL, 0 },
+	{ "a='b'", "a", "b", 0 },
+	{ "a='b", "a", "b", 0 },
+	{ "a=b'", "a", "b'", 0 },
+	{ "a=\"b\"", "a", "b", 0 },
+	{ "a=b\"", "a", "b\"", 0 },
+	{ "a=\"b", "a", "b", 0 },
+	{ "a=b # comment", "a", "b # comment", 0 }, // Maybe fix this.
+	{ "field=longvalue with spaces", "field", "longvalue with spaces", 0 },
+	{ "test:=reset", "test", "reset", 1 },
 };
 
 START_TEST(test_conf_get_pair)
 {
-        FOREACH(d)
+	FOREACH(d)
 	{
 		char *field=NULL;
 		char *value=NULL;
 		char *str=strdup(d[i].str);
-		conf_get_pair(str, &field, &value);
+		int reset=0;
+		conf_get_pair(str, &field, &value, &reset);
 		if(!field || !d[i].field)
 			fail_unless(field==d[i].field);
 		else
@@ -77,31 +80,12 @@ START_TEST(test_conf_get_pair)
 			fail_unless(value==d[i].value);
 		else
 			fail_unless(!strcmp(value, d[i].value));
+		fail_unless(d[i].reset==reset);
 		free(str);
 	}
 }
 END_TEST
 
-START_TEST(test_client_conf)
-{
-	struct conf **confs=NULL;
-	setup(&confs, NULL);
-	build_file(CONFFILE, MIN_CLIENT_CONF);
-	fail_unless(!conf_load_global_only(CONFFILE, confs));
-	fail_unless(get_e_burp_mode(confs[OPT_BURP_MODE])==BURP_MODE_CLIENT);
-	ck_assert_str_eq(get_string(confs[OPT_SERVER]), "4.5.6.7");
-	ck_assert_str_eq(get_string(confs[OPT_PORT]), "1234");
-	ck_assert_str_eq(get_string(confs[OPT_STATUS_PORT]), "12345");
-	ck_assert_str_eq(get_string(confs[OPT_LOCKFILE]), "/lockfile/path");
-	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT]), "/ssl/cert/path");
-	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT_CA]), "/cert_ca/path");
-	ck_assert_str_eq(get_string(confs[OPT_SSL_PEER_CN]), "my_cn");
-	ck_assert_str_eq(get_string(confs[OPT_SSL_KEY]), "/ssl/key/path");
-	ck_assert_str_eq(get_string(confs[OPT_CA_CSR_DIR]), "/csr/dir");
-	tear_down(NULL, &confs);
-}
-END_TEST
-
 static void assert_strlist(struct strlist **s, const char *path, int flag)
 {
 	if(!path)
@@ -114,6 +98,146 @@ static void assert_strlist(struct strlist **s, const char *path, int flag)
 	*s=(*s)->next;
 }
 
+static void check_ports(struct conf **confs,
+	const char *port, int max_children,
+	const char *status_port, int max_status_children)
+{
+	struct strlist *s;
+	s=get_strlist(confs[OPT_PORT]);
+	assert_strlist(&s, port, max_children);
+	s=get_strlist(confs[OPT_STATUS_PORT]);
+	assert_strlist(&s, status_port, max_status_children);
+}
+
+START_TEST(test_client_conf)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_CLIENT_CONF);
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+	fail_unless(get_e_burp_mode(confs[OPT_BURP_MODE])==BURP_MODE_CLIENT);
+	ck_assert_str_eq(get_string(confs[OPT_SERVER]), "4.5.6.7");
+	check_ports(confs, "1234", 0, "12345", 0);
+	ck_assert_str_eq(get_string(confs[OPT_LOCKFILE]), "/lockfile/path");
+	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT]), "/ssl/cert/path");
+	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT_CA]), "/cert_ca/path");
+	ck_assert_str_eq(get_string(confs[OPT_SSL_PEER_CN]), "my_cn");
+	ck_assert_str_eq(get_string(confs[OPT_SSL_KEY]), "/ssl/key/path");
+	ck_assert_str_eq(get_string(confs[OPT_CA_CSR_DIR]), "/csr/dir");
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+static void check_client_ports(struct conf **confs,
+	int p_backup, int p_restore, int p_verify, int p_list, int p_delete)
+{
+	fail_unless(get_int(confs[OPT_PORT_BACKUP])==p_backup);
+	fail_unless(get_int(confs[OPT_PORT_RESTORE])==p_restore);
+	fail_unless(get_int(confs[OPT_PORT_VERIFY])==p_verify);
+	fail_unless(get_int(confs[OPT_PORT_LIST])==p_list);
+	fail_unless(get_int(confs[OPT_PORT_DELETE])==p_delete);
+}
+
+START_TEST(test_client_conf_monitor_exe)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_CLIENT_CONF
+		"monitor_exe=/some/path"
+	);
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+	ck_assert_str_eq(get_string(confs[OPT_MONITOR_EXE]), "/some/path");
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_client_conf_ports_opt_port_only)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_CLIENT_CONF);
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+	check_client_ports(confs, 1234, 1234, 1234, 1234, 1234);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_client_conf_ports_opt_port_and_restore)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_CLIENT_CONF_NO_PORTS
+		"port=1234\n"
+		"port_restore=5555\n"
+	);
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+	check_client_ports(confs, 1234, 5555, 5555, 1234, 1234);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_client_conf_ports_opt_port_and_all)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_CLIENT_CONF_NO_PORTS
+		"port=1234\n"
+		"port_backup=2345\n"
+		"port_restore=3456\n"
+		"port_verify=4567\n"
+		"port_list=5678\n"
+		"port_delete=6789\n"
+	);
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+	check_client_ports(confs, 2345, 3456, 4567, 5678, 6789);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_client_conf_ports_no_opt_port_and_all)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_CLIENT_CONF_NO_PORTS
+		"port_backup=2345\n"
+		"port_restore=3456\n"
+		"port_verify=4567\n"
+		"port_list=5678\n"
+		"port_delete=6789\n"
+	);
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+	check_client_ports(confs, 2345, 3456, 4567, 5678, 6789);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_client_conf_ports_none)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_CLIENT_CONF_NO_PORTS);
+	fail_unless(conf_load_global_only(CONFFILE, confs)==-1);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_client_conf_ca_conf_problems)
+{
+	const char *buf="mode=client\n"
+		"server=4.5.6.7\n"
+		"port=1234\n"
+		"status_port=12345\n"
+		"lockfile=/lockfile/path\n"
+		"ca_burp_ca=blah\n"
+	;
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, buf);
+	fail_unless(conf_load_global_only(CONFFILE, confs)==-1);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
 static void assert_include(struct strlist **s, const char *path)
 {
 	assert_strlist(s, path, 1);
@@ -242,8 +366,7 @@ START_TEST(test_server_conf)
 	build_file(CONFFILE, MIN_SERVER_CONF);
 	fail_unless(!conf_load_global_only(CONFFILE, confs));
 	fail_unless(get_e_burp_mode(confs[OPT_BURP_MODE])==BURP_MODE_SERVER);
-	ck_assert_str_eq(get_string(confs[OPT_PORT]), "1234");
-	ck_assert_str_eq(get_string(confs[OPT_STATUS_PORT]), "12345");
+	check_ports(confs, "1234", 5, "12345", 5);
 	ck_assert_str_eq(get_string(confs[OPT_LOCKFILE]), "/lockfile/path");
 	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT]), "/ssl/cert/path");
 	ck_assert_str_eq(get_string(confs[OPT_SSL_CERT_CA]), "/cert_ca/path");
@@ -258,6 +381,89 @@ START_TEST(test_server_conf)
 }
 END_TEST
 
+START_TEST(test_server_port_conf_one_max_child)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
+		"port=1234\n"
+		"status_port=12345\n"
+		"max_children=12\n"
+		"max_status_children=21\n");
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+	check_ports(confs, "1234", 12, "12345", 21);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_server_port_conf_too_many_max_child)
+{
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
+		"port=1234\n"
+		"max_children=12\n"
+		"max_children=21\n");
+	fail_unless(conf_load_global_only(CONFFILE, confs)==-1);
+	fail_unless(get_strlist(confs[OPT_STATUS_PORT])==NULL);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_server_port_conf_few_max_child)
+{
+	struct strlist *s;
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
+		"port=1234\n"
+		"port=2345\n"
+		"port=3456\n"
+		"max_children=12\n"
+		"max_children=21\n");
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+
+	s=get_strlist(confs[OPT_PORT]);
+	assert_strlist(&s, "1234", 12);
+	assert_strlist(&s, "2345", 21);
+	assert_strlist(&s, "3456", 21); // takes the previous as default
+	fail_unless(get_strlist(confs[OPT_STATUS_PORT])==NULL);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
+START_TEST(test_server_port_conf_complex)
+{
+	struct strlist *s;
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
+		"port=1234\n"
+		"port=2345\n"
+		"port=3456\n"
+		"max_children=12\n"
+		"max_children=21\n"
+		"max_children=37\n"
+		"status_port=987\n"
+		"status_port=654\n"
+		"status_port=321\n"
+		"max_status_children=4\n"
+		"max_status_children=5\n"
+		"max_status_children=6\n");
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+
+	s=get_strlist(confs[OPT_PORT]);
+	assert_strlist(&s, "1234", 12);
+	assert_strlist(&s, "2345", 21);
+	assert_strlist(&s, "3456", 37);
+	s=get_strlist(confs[OPT_STATUS_PORT]);
+	assert_strlist(&s, "987", 4);
+	assert_strlist(&s, "654", 5);
+	assert_strlist(&s, "321", 6);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
 static void pre_post_assertions(struct conf **confs, const char *pre_path,
 	const char *post_path, const char *pre_arg1, const char *pre_arg2,
 	const char *post_arg1, const char *post_arg2,
@@ -498,6 +704,7 @@ START_TEST(test_clientconfdir_conf)
 		"timer_arg=/timer/arg1\n"
 		"timer_arg=/timer/arg2\n"
 		"label=qwe\n"
+		"client_lockdir=/tmp/blah\n"
 		NOTIFY_CONF
 	;
 	const char *buf=MIN_CLIENTCONFDIR_BUF
@@ -510,6 +717,7 @@ START_TEST(test_clientconfdir_conf)
 		"keep=7\n"
 		"working_dir_recovery_method=resume\n"
 		"librsync=0\n"
+		"librsync_max_size=10Mb\n"
 		"version_warn=0\n"
 		"path_length_warn=0\n"
 		"syslog=1\n"
@@ -546,6 +754,7 @@ START_TEST(test_clientconfdir_conf)
 	fail_unless(get_e_recovery_method(
 	  cconfs[OPT_WORKING_DIR_RECOVERY_METHOD])==RECOVERY_METHOD_RESUME);
 	fail_unless(get_int(cconfs[OPT_LIBRSYNC])==0);
+	fail_unless(get_uint64_t(cconfs[OPT_LIBRSYNC_MAX_SIZE])==10485760);
 	fail_unless(get_int(cconfs[OPT_VERSION_WARN])==0);
 	fail_unless(get_int(cconfs[OPT_PATH_LENGTH_WARN])==0);
 	fail_unless(get_int(cconfs[OPT_SYSLOG])==1);
@@ -563,6 +772,8 @@ START_TEST(test_clientconfdir_conf)
 	assert_strlist(&s, "rty", 0);
 	assert_strlist(&s, "xyz", 0);
 	notify_assertions(cconfs);
+	ck_assert_str_eq(get_string(globalcs[OPT_CLIENT_LOCKDIR]), "/tmp/blah");
+	ck_assert_str_eq(get_string(cconfs[OPT_CLIENT_LOCKDIR]), "/tmp/blah");
 	tear_down(&globalcs, &cconfs);
 }
 END_TEST
@@ -616,6 +827,27 @@ START_TEST(test_clientconfdir_extra)
 }
 END_TEST
 
+START_TEST(test_strlist_reset)
+{
+	struct strlist *s;
+	struct conf **confs=NULL;
+	setup(&confs, NULL);
+	build_file(CONFFILE, MIN_SERVER_CONF_NO_PORTS
+		"port=1234\n"
+		"timer_arg = /ignored/timer/arg1\n"
+		"timer_arg = /ignored/timer/arg2\n"
+		"timer_arg := /timer/arg3\n"
+		"timer_arg = /timer/arg4\n");
+	fail_unless(!conf_load_global_only(CONFFILE, confs));
+
+	s=get_strlist(confs[OPT_TIMER_ARG]);
+	assert_strlist(&s, "/timer/arg3", 0);
+	assert_strlist(&s, "/timer/arg4", 0);
+	assert_include(&s, NULL);
+	tear_down(NULL, &confs);
+}
+END_TEST
+
 START_TEST(test_clientconfdir_server_script)
 {
 	struct conf **globalcs=NULL;
@@ -695,10 +927,21 @@ Suite *suite_conffile(void)
 
 	tcase_add_test(tc_core, test_conf_get_pair);
 	tcase_add_test(tc_core, test_client_conf);
+	tcase_add_test(tc_core, test_client_conf_ca_conf_problems);
 	tcase_add_test(tc_core, test_client_includes_excludes);
 	tcase_add_test(tc_core, test_client_include_failures);
 	tcase_add_test(tc_core, test_client_include_glob);
+	tcase_add_test(tc_core, test_client_conf_monitor_exe);
+	tcase_add_test(tc_core, test_client_conf_ports_opt_port_only);
+	tcase_add_test(tc_core, test_client_conf_ports_opt_port_and_restore);
+	tcase_add_test(tc_core, test_client_conf_ports_opt_port_and_all);
+	tcase_add_test(tc_core, test_client_conf_ports_no_opt_port_and_all);
+	tcase_add_test(tc_core, test_client_conf_ports_none);
 	tcase_add_test(tc_core, test_server_conf);
+	tcase_add_test(tc_core, test_server_port_conf_one_max_child);
+	tcase_add_test(tc_core, test_server_port_conf_too_many_max_child);
+	tcase_add_test(tc_core, test_server_port_conf_few_max_child);
+	tcase_add_test(tc_core, test_server_port_conf_complex);
 	tcase_add_test(tc_core, test_server_script_pre_post);
 	tcase_add_test(tc_core, test_server_script);
 	tcase_add_test(tc_core, test_backup_script_pre_post);
@@ -707,6 +950,7 @@ Suite *suite_conffile(void)
 	tcase_add_test(tc_core, test_restore_script);
 	tcase_add_test(tc_core, test_clientconfdir_conf);
 	tcase_add_test(tc_core, test_clientconfdir_extra);
+	tcase_add_test(tc_core, test_strlist_reset);
 	tcase_add_test(tc_core, test_clientconfdir_server_script);
 	tcase_add_test(tc_core, test_conf_switch_to_orig_client_fail);
 	tcase_add_test(tc_core, test_conf_switch_to_orig_client_ok);
